package PureMessage::DataStore;


sub new {
  my $class = shift;
  my %options = @_;
  my ($type,$name) = split(':',$options{store});
  die "Invalid store '$options{store}'" unless $type and $name;
  my $data_store;
  my $package = "PureMessage::DataStore::$type";
  $data_store = eval "require $package; $package->new($name,\%options)";
  die "Invalid store type '$type': $@." unless $data_store;
  
  my %hash = ( ds => $data_store,
               store => $options{store},
             );

  my $self = bless \%hash, $class;
  return $self;

}


sub keys {
  my $self = shift;
  return $self->{ds}->keys(@_);
}

sub get {
  my $self = shift;
  return $self->{ds}->get(@_);
}

sub delete {
  my $self = shift;
  return $self->{ds}->delete(@_);
}

sub clean {
  my $self = shift;
  foreach my $key ($self->keys) {
    $self->delete($key);
  }
}



1;


=head1 NAME

PureMessage::DataStore 

=head1 SYNOPSIS

    use PureMessage::DataStore;
    
    my $ds = PureMessage::DataStore->new( store => 'dir:my_store');
    
    foreach my $key ($ds->keys) {
      my $store = $ds->get($key);
      $store->store('key','value');
      $store->retrieve('key');
      my @contents = $store->contents();
      print "Store '$key' has the following contents:\n";
      print "\t$_\n" foreach @contents;
    }
    

=head1 DESCRIPTION

The C<PureMessage::DataStore> implements an interface to an
abstract store of key value pairs.

=over 4


=item B<store>

Returns a new PureMessage::DataStore object. This is a collection
of individual stores of data indexed by key.

=item B<delete> I<$KEY>

Deletes the store $KEY and all it's contents.

=item B<clean>

Will remove B<ALL> stores and their keys from the datastore. 

=item B<get> I<$KEY>

Returns the key specific store. This is the object that all data
manipulations should be done on. The returned object can be used
to manipulate the store data with the following methods.

=over 4

=item *

$store->contents

In an array context will return an array containing the keys that
exist in that store, otherwise it will return an array ref if the
same contents.

=item *

$store->store($key,$DATA)

Will store the passed value into the store under the $key. This
can later be retrieved using $store->retrieve($key).

=item *

$store->retrieve($key)

Will retrieve the data associated with that key from the store.

=item *

$store->delete($key)

Will delete the passed key name from the store.

=item *

$store->store_seq($seq_key,$DATA)

Will store the data into an autoincremented key prefixed by I<$seq_key>. 
The keys are guaranteed to be unique and created sequentially.  The
specific keys created can be returned later using the 
$store->contents_seq command.

=item *

$store->contents_seq($seq_key)

In an array context will return the sequential keys that match the
passed $seq_key, otherwise returns an array ref.

=back

=cut





