package PureMessage::DataStore::dir;
use PureMessage::Config qw( %config );
use PureMessage::Version qw( $PREFIX );
use File::Path qw(mkpath);
use Exporter;
use strict;
our @ISA = qw( Exporter);
our @EXPORT_OK = qw( key2file file2key );

sub new {
  my ($class,$name,%options) = @_;
  
  my $dir = exists($options{dir}) ? "$options{dir}/$name"
                                  : "$PREFIX/var/datastore/$name";
  
  die "Unable to create store dir '$dir'" 
    unless -d $dir or mkpath($dir);
  
  my %hash = ( name => $name,
               dir => $dir,
             );

  my $self = bless \%hash, $class;
  return $self;
}

sub keys {
  my $self = shift;
  opendir DIR, $self->{dir} or die "$self->{dir}: $!";
  my @keys = map (file2key($_), grep($_ !~ /^\.\.?$/, readdir DIR)); 
  closedir DIR;
  return @keys; 
}

sub delete {
  my $self = shift;
  my $key = shift;
  return unless -e $self->{dir} and -d _;
  my $store = $self->get($key);
  $store->delete($_) foreach $store->contents();
  my $key_dir = $self->{dir} . '/' . key2file($key);
  die "Unable to remove key dir '$key_dir' for key '$key': $!" unless rmdir($key_dir);
}

sub get {
  my $self = shift;
  my $key = shift;
  my $dir = key2file($key,$self->{dir});
  return PureMessage::Data::Store::dir::data->new($dir);
}

# this code borrowed from the List::Utils package
# a quoted-printable-like codec to ensure filesystem-friendly keys
sub key2file {
    my $key = shift;
    my $path = shift;
    $key =~ s/([^\w\@\-\.])/sprintf("=%02X", ord($1))/eg;
    return defined $path ? "$path/$key" : $key;
}

sub file2key {
    my $file = shift;
    $file =~ s,.*/,,;
    $file =~ s/=([\da-fA-F]{2})/pack("C", hex($1))/ge;
    return $file;
}

package PureMessage::Data::Store::dir::data;
use ActiveState::File::Atomic;
use File::Path qw(mkpath);

*key2file = \&PureMessage::DataStore::dir::key2file;
*file2key = \&PureMessage::DataStore::dir::file2key;

sub new {
  my ($class,$dir,%options) = @_;
  my %hash = ( dir => $dir,
             );
 
  my $self = bless \%hash, $class;
  return $self;
}

sub _init_dir {
  my $self = shift;
  unless (-d $self->{dir}) {
    die "Unable to create path '$self->{dir}'" 
        unless mkpath($self->{dir});
  }
}

sub store {
  my $self = shift;
  my ($key,$value) = @_;
  $self->_init_dir();
  my $file = ActiveState::File::Atomic->new(key2file($key,$self->{dir}),writable => 1, create => 1);
  $file->commit_string($value);
  $key;
}


# store data under a sequentially created unique key. The generated keys are 
# guaranteed to be unique, with their order of creation easily determined by a 
# lexical sort. **note** this will clobber any keys already stored that have
# the same name as 'key_prefix'!!
#
#   a good idiom is 
#   $data->store_seq('key_prefix',$_) foreach qw( a b c d e f g h );
#   my $keys = sort grep /^key_prefix/, $data->contents();

sub store_seq {
  my $self = shift;    
  my ($k_prefix,$value) = @_;
  $self->_init_dir();
  my $file = ActiveState::File::Atomic->new(key2file($k_prefix,$self->{dir}),writable => 1, create => 1);
  my $contents = $file->slurp;
  $contents =~ s/\s//g;
  die "Contents of $k_prefix is not numeric\n" unless $contents =~ /^\d+$/ or $contents eq '';
  $contents = 0 unless $contents; # default to zero if there's no value
  $contents++;
  my $filename = sprintf("${k_prefix}_%06d",$contents);
  my $seq_file = ActiveState::File::Atomic->new(key2file($filename,$self->{dir}),writable => 1,create => 1);
  $seq_file->commit_string($value);
  $file->commit_string($contents);
  return $filename;
}

# return the contents that matches the sequence name
sub contents_seq {
  my ($self,$seq_key) = @_;
  return unless -d $self->{dir} or not defined($seq_key);
  my @contents = sort grep { /^${seq_key}_/ } $self->contents;
  return @contents if wantarray;
  return \@contents;
}

# returns the available "keys" in this data store
sub contents {
  my $self = shift;
  return unless -d $self->{dir};
  my @files;
  foreach my $file (glob("$self->{dir}/* ")) {
      $file =~ s/^$self->{dir}//; # remove the dir part if it's there
      $file =~ s#^/##; # remove any leading slashes that might be remaining
      $file = file2key($file);
      push(@files,$file);
  }
  return @files if wantarray;
  return \@files;
}

sub delete {
   my $self = shift;
   my $key = shift;
   my $absfile = key2file($key,$self->{dir});
   die "File not found '$key'" unless -e $absfile and -r _;
   unlink $absfile;
}

sub retrieve {
  my $self = shift;
  my ($key) = @_;
  my $file_name = key2file($key,$self->{dir});
  return unless -e $file_name;
  my $file = ActiveState::File::Atomic->new($file_name);
  return $file->slurp();
}


1;





