package PureMessage::HashDB;
use strict;
use warnings;
use Carp;
use CDB_File;
use ActiveState::File::Atomic;
use PureMessage::_sys;

sub DELIM()        { "\t" }
sub DB_EXTENSION() { ".cdb" }
sub LOCK_TIMEOUT() { 5 }

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my %opts = @_;

    my $file     = delete $opts{file};
    my $writable = delete $opts{writable};
    my $create   = delete $opts{create};
    my $timeout  = delete $opts{lock_timeout} || LOCK_TIMEOUT;
    my $delim    = delete $opts{delimiter}    || DELIM;
    my $db_ext   = delete $opts{db_extension} || DB_EXTENSION;

    if ($^W) {
	warn "Invalid option '$_'" foreach keys %opts;
    }

    # Create implies writable
    $writable = 1 if $create;

    # Lock out other writers. 
    my $wr_lock;
    if ($writable) {
	$wr_lock = eval { 
	    ActiveState::File::Atomic->new(
		$file . ".lock",
		writable => 1,
		create   => 1,
		timeout  => $timeout) 
	};
	croak "Failed to lock $file for writing: $@" if $@;
    }
    
    my $hash;
    my $db_name = $file . $db_ext;
    if (-e $db_name) {
	$hash = CDB_File->TIEHASH($db_name) or
	    croak "Failed to open $db_name: $!";
    }
    my $db_mtime = (stat($db_name))[9];
	
    return bless {
	writable  => $writable,
	create    => $create,
	file      => $file,
	db_name   => $db_name,
	hash      => $hash,
	wr_lock   => $wr_lock,
	delim     => $delim,
	db_ext    => $db_ext,
	db_mtime  => $db_mtime,
    }, $class;
}

sub put {
    my ($self, $key, $val) = @_;
    croak "Can't call 'update' on a read-only object." 
	unless $self->{writable};

    delete $self->{deleted}{$key} if exists $self->{deleted}{$key};
    return $self->{changed}{$key} = $val if 
	defined $self->{hash} && defined $self->{hash}->FETCH($key);
    return $self->{new}{$key} = $val;
}

sub get {
    my ($self, $key) = @_;

    return undef if exists $self->{deleted}{$key};
    return $self->{changed}{$key} if exists $self->{changed}{$key};
    return $self->{new}{$key}     if exists $self->{new}{$key};
    return $self->{hash}->FETCH($key) if $self->{hash};
}

sub del {
    my ($self, $key) = @_;
    croak "Can't call 'delete' on a read-only object." 
	unless $self->{writable};

    $self->{deleted}{$key}++;
}

sub enumerate {
    my ($self, $cb) = @_;

    my $deleted = $self->{deleted};
    my $changed = $self->{changed};
    my $new     = $self->{new};
    if (my $hash = $self->{hash}) {
	my $k;
	for ($k = $hash->FIRSTKEY; defined $k; $k = $hash->NEXTKEY($k)){
	    next if exists $deleted->{$k};
	    if (exists $changed->{$k}) {
		&$cb($k, $changed->{$k});
	    }
	    else {
		&$cb($k, $hash->FETCH($k));
	    }
	}
    }

    foreach my $k(keys %$new) {
	&$cb($k, $new->{$k});
    }
}

sub save {
    my $self = shift;
    my %opt = @_;
    $opt{commit_db} = 1 unless defined $opt{commit_db};
    
    croak "Can't call 'save' on a read-only object." 
	unless $self->{writable};

    my $delim   = $self->{delim};
    my $changed = $self->{changed};
    my $deleted = $self->{deleted};
    my $new     = $self->{new};
    
    # Copy the contents of the source file to the temp file. Replace any rows
    # who's key is in the changed hash and skip any row that is in the deleted
    # hash. Skip empty lines and comments. Any values remaining in changed
    # are new rows to add.
    my $src = eval {
	    ActiveState::File::Atomic->new(
	        $self->{file},
	        writable => 1,
	        ( $self->{create} ? (create => $self->{create}) : () )
        )
    };
    die "Failed to open $self->{file}: $@" if $@;
    my $dst = $src->tempfile;

    while (defined($_ = $src->readline)) {
	chomp;
	s/^\s*//;
	s/\s*$//;
	next unless length;
	next if /^#/;
	my ($k, $v) = split($delim, $_, 2);
	if (exists $deleted->{$k}) {
	    next;
	}
	elsif (exists $changed->{$k}) {
	    print $dst $k, $delim, $changed->{$k}, "\n";
	    delete $changed->{$k};
	}
	else {
	    print $dst $k, $delim, $v, "\n";
	}
    }

    foreach my $k (keys %$new) {
	print $dst $k, $delim, $new->{$k}, "\n";
    }

    # Close and commit the new text file and the database file. Reload
    # The DB handle when we're all done.
    $src->commit_tempfile;

    if ($opt{commit_db}) {
	$self->commit_db();
	$self->reload() if $opt{reload};
    }
}

sub commit_db {
    my $self = shift;

    open(my $src, "<$self->{file}") or die "Failed to open $self->{file}: $!";
    
    my $dst = CDB_File->new($self->{db_name}, 
	$self->{db_name} . PureMessage::_sys::tid())
	or die "Failed to open the database for writing: $!";

    while (<$src>) {
	chomp;
	s/^\s*//;
	s/\s*$//;
	next unless length;
	next if /^#/;
	my ($k, $v) = split($self->{delim}, $_, 2);
	$dst->insert($k, $v);
    }
    $dst->finish;
}

sub reload {
    my $self = shift;
    croak "Can't call 'reload' on an writable object." if $self->{writable};

    delete $self->{hash};
    if (-e $self->{db_name}) {
        $self->{hash} = CDB_File->TIEHASH($self->{db_name}) or 
	    die "Failed to open $self->{db_name}: $!";
    }
}

sub DESTROY {
    my $self = shift;
    delete $self->{wr_lock};

    my $lf = "$self->{file}.lock";
    unlink($lf) if -e $lf;
}

1;

__END__

=head1 PureMessage::HashDB

Interface to a file-based hash table.

=head1 DESCRIPTION

Wraps CDB_File to provide a simple interface that supports
both updating and reading. All data is recorded to a text
file in a human readable format before getting compiled into
the binary database.

Readers don't block writers and writers don't block readers.

=head1 SEE ALSO

L<CDB_File>
