package PureMessage::Lang;

=head1 NAME

PureMessages::Lang - Get the installed language translations

=head2 SYNOPSIS

    use PureMessage::Lang qw(get_langs get_eng_langs get_trans_langs get_lang_map);

    set_lang_dir("/path/to/lang/files");

    my @langs = get_langs();
    my @trans_langs = get_trans_langs();
    my %lang_map = get_lang_map();

=cut

use strict;
use Exporter;
use Carp;
use PureMessage::Version qw($PREFIX);
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(set_lang_dir get_langs get_eng_langs get_trans_langs get_lang_map);

our $lang_dir; # $PREFIX/etc/manager/lang/installed

sub _check_dir {
    if (!$lang_dir) {
        croak("language directory has not been set!");
    }
    if (! -e $lang_dir) {
        croak("\"$lang_dir\" does not exist!");
    }
    if (! -d $lang_dir) {
        croak("\"$lang_dir\" is not a directory!");
    }
    if (! -r $lang_dir) {
        croak("Cannot read \"$lang_dir\"!");
    }
}

sub _get_dir_files {
    _check_dir;
    
    if (! opendir(LANG_DIR, $lang_dir)) {
        croak("Cannot open \"$lang_dir\"!");
    }
    my @lang_files = grep(!/^\.\.?$/, readdir(LANG_DIR));
    return @lang_files;
}

sub _check_file {
    my $file = shift;
    if (! -e "$lang_dir/$file") {
        croak("\"$lang_dir/$file\" does not exist!");
    }
    if (! -r "$lang_dir/$file") {
        croak("Cannot read \"$lang_dir/$file\"!");
    }
    return 1;
}

sub _read_lang_files {
    my @lang_files = @_;

    # language abbreviations, English Languages, Translated Languages
    my (@langs, @eng_langs, @trans_langs);
    my %lang_map;
    foreach my $file (@lang_files) {
	_check_file($file);
	open(FILE, "<$lang_dir/$file") or croak("Cannot open \"$lang_dir/$file\"!");
	push(@langs, $file);
	my $line = <FILE>;
	chomp $line;
	my ($eng_lang, $trans_lang) = split(',', $line);
	push(@eng_langs, $eng_lang);
	push(@trans_langs, $trans_lang);
	$lang_map{"$file"} = {'eng_lang' => $eng_lang, 'trans_lang' => $trans_lang};
    }
    my %lang_info = ('lang_map' => \%lang_map,
		     'langs' => \@langs,
		     'eng_langs' => \@eng_langs,
		     'trans_langs' => \@trans_langs,
		     );
    return \%lang_info;
}

=item set_lang_dir("/path/to/langs")

Set the path to the directory which contains a list of installed languages.

=cut

sub set_lang_dir($) {
    my $dir = shift;
    if(substr($dir, 0, 1) eq '/') {
        $lang_dir = $dir;
    } else {
        $lang_dir = "$PREFIX/$dir";
    }

    return $lang_dir;
}

=item get_langs()

This will return a list of abbreviations of the installed language translations.

=cut

sub get_langs() {
    my $lang_info = _read_lang_files(_get_dir_files());
    my $langs = $lang_info->{'langs'};
    return @$langs;
}
=item get_eng_langs()

This will return an English list of the installed language translations.

=cut

sub get_eng_langs() {
    my $lang_info = _read_lang_files(_get_dir_files());
    my $eng_langs = $lang_info->{'eng_langs'};
    return @$eng_langs;
}

=item get_trans_langs()

This will return a translated list of the installed language translations.

=cut

sub get_trans_langs() {
    my $lang_info = _read_lang_files(_get_dir_files());
    my $trans_langs = $lang_info->{'trans_langs'};
    return @$trans_langs;
}

=item get_lang_map()

This will return a hash with the two letter language abbreviations as keys.
The keys associate to hash references.  These hash references have two keys,
"lang" and "trans_lang" which corespond to the English name for the language
and the translated name for the language.

    E.g.:
    my %map = ( 'en' => { 'eng_lang' => 'English',
			  'trans_lang' => 'English' },
		'fr' => { 'eng_lang' => 'French',
			  'trans_lang' => 'Francais' },
		'de' => { 'eng_lang' => 'German',
			  'trans_lang' => 'Dutsche' },
	      );

=cut

sub get_lang_map() {
    my $lang_info = _read_lang_files(_get_dir_files());
    my $lang_map = $lang_info->{'lang_map'};
    return %$lang_map;
}

1;
