package PureMessage::List;

use strict;
use base 'PureMessage::List::Base';

sub _kind { "list" }

sub match {
    my($self, @words) = @_;
    return 0;
}

sub kmatch {
    my $self = shift;
    my $key = shift;
    return $self->match(@_);    
}

sub matcher {
    my $self = shift;
    return sub {
	$self->match(@_);
    }
}

1;

__END__

=head1 NAME

PureMessage::List - Interface to lists

=head1 SYNOPSIS

  use PureMessage::List;

  my $list = PureMessage::List->new("blacklist");
  if ($list->match('foo@bar.com')) {
      #...
  }

=head1 DESCRIPTION

The purpose of a C<PureMessage::List> object is allow strings to be
matched against it.  It is called I<list> because it is used to
represent whitelists, blacklists and opt-in lists for email addresses
and relays, but the way list object are used is basically the same as
matching against regular expressions in Perl.  Matching against a list
is performed with the match() method, where the strings to be matched
is provided as arguments.

C<PureMessage::List> objects are designed so that they can implement
matching operations as specified by the Sieve mail filtering language
(RFC 3028).  They expose the concept of I<match_type> and
I<comparator> as specified in Sieve.  List can also be interfaces to
external databases.

The following class methods are provided:

=over

=item PureMessage::List->Config( %opts )

This returns an C<ActiveState::Config> parser object for the
F<lists.conf> configuration file.  This can be used to
programmatically inspect or modify the configuration.  See
L</"Configuration file">.

=item PureMessage::List->Available

=item PureMessage::List->Available( $id )

This returns a list of hash references that describe the predefined
lists available on the system. Predefined lists are set up in the
F<lists.conf> configuration file.  If an $id argument is provided,
only information about the given list this is returned.  If no such
list exists C<undef> is returned.  If no $id argument is provided then
the number of available lists are returned in scalar context.

Each hash returned contains the following entries:

  id
  name
  description
  precious
  source

The C<id> value is used to instantiate a new list object that can then
be used for matching.  The C<name> and C<description> values are
provided for presentation in user interfaces.

The C<precious> value is a boolean.  If it is true than this is a list
that the system depends on the presence of.  User interfaces should not
allow this list to be deleted or renamed.

The C<source> is a normalized list source specifier.  It will always
be of the form C<scheme:stuff>.  If the I<scheme> is C<file>, then the
I<stuff> is a file name that can be passed to the
PureMessage::ListFile constructor to examine or modify the source
file.  The source is C<inline:> if list items are inlined using an
<items> section in F<lists.conf>.

The attributes C<match_type>, C<comparator> and C<mtime> might be there 
if they apply to the given list.

This shows an example how this can be used.  Here we a append the
entry "example.com" to all the list files (this is not really a
wise thing to do):

    for (PureMessage::List->Available) {
	my $source = $_->{source};
	if ($source =~ s/^file://) {
	    my $f = PureMessage::ListFile->new($source, writable => 1);
	    $f->append("example.com");
	    $f->commit;
	}
    }

=item PureMessage::List->Setup( %opts )

This will set up a new predefined list in the F<lists.conf>
configuration file.  The new list is described using key/value pairs
which are basically the same as those allowed in the configuration
file.  It is mandatory to provide an C<id> argument.

There is no return value.  The method will croak if it is not possible
to set up the new list.  It is not possible to setup a new list with
the same C<id> as an existing list.

=item $list = PureMessage::List->new( $id )

=item $list = PureMessage::List->new( %opts )

The list constructor.  It is either called with a list id or key/value
pairs as argument.  If an id is provided, then the list parameters are
picked up from the F<lists.conf> configuration file. Otherwise
key/value pairs are basically the same as the those allowed in the
configuration file. If an id is provided then the same list object
might be returned each time the constructor is called.

=back

The following methods are provided by a C<PureMessage::List> object:

=over

=item $list->id

Returns the list id for the list (or C<undef> if this is not a list
defined in F<lists.conf>).

=item $list->name

Returns the name of a list. The name is suitable for display.

=item $list->description

Returns a short description of the purpose of the list.

=item $list->match_type

Returns the match_type for the current list. This attribute is always
defined, and default to "is".  See L</"Match type">.

=item $list->comparator

Returns the name of the comparator function to use.  See
L</"Comparator">.

=item $list->source

Returns the list source.

=item $list->match( $string, ... )

The match method returns true iff any of the strings provided as
argument match against the list.

=item $list->matcher

The matcher method returns a function.  This function works like the
match method above, it returns true iff any of the strings provided as
its arguments match.

Example usage:

   # set up blacklisted function
   *blacklisted = PureMessage::List->new("blacklist")->matcher;

   # use it
   die if blacklisted("foo.com");

=item $list->refresh

If the list source is a file that has changed since list construction
or last refresh, then reinitialize the list matcher based on the new
content.  The return value is true iff the list was refreshed.

Note that the function returned by the matcher() method is not
automatically refreshed.  A new matcher should be obtained by the
matcher() method after refresh has returned true.

=item $list->mtime

For file and cdbfile list types, returns the file modification time in 
seconds from epoch time.

=back

=head2 Configuration file

The F<lists.conf> configuration file describes the available 
predefined named lists.  The file contains zero or more named C<list>
sections.  This is an example of such a section:

  <list foo>
    name = "The Foo List"
    description = "Just an example"
    precious = yes
    match_type = mail
    source = file:foo
  </list>

The list id in this case is C<foo>, the name is C<The Foo List>,
similar for the other attributes, and finally the items for the list
are picked up from the file F<foo>.

The C<precious> field is a boolean attribute that tells the 
user interface that it should not be allowed to delete the 
entry or modify matching parameters (match_type/comparator).  
It is used to differentiate between lists distributed with 
PureMessage and custom lists defined by users.

The C<name>, C<description>, C<match_type> and C<comparator> fields
just encode the value for the given attribute.  The C<name> will
default to the list id.  The C<match_type> will default to C<is>.

The mutually exclusive C<items> and C<source> fields define how the
items of the list are obtained.  The C<items> are used to inline list
items directly:

  <list foo>
     <items>
        a
        b
     </items>
  </list>

The C<source> is prefixed by a URI-style scheme that indicates 
the source type.  It can be one of the following:

=over

=item C<file:>

Items are read from the indicated file.

If the source field does not begin with a scheme part, it is 
treated as if it was prefixed with "file:".

=item C<dbfile:>

Look up matches against a Berkeley database file, see L<DB_File>.  If
the comparator is specified as "i;ascii-casemap" (the default), then
keys are downcased before they are looked up.  A match is successful
if the match key exists in the database.  The corresponding value is
always ignored.

=item C<cdbfile:>

Look up matches against a CDB file, see L<CDB_File>.
If the comparator is specified as "i;ascii-casemap" (the default), then
keys are downcased before they are looked up. cdbfile lists
and maps are compiled from regular text-file list and maps; see
L<pmx-makemap> for more information.

=item C<dbi:>

Look up matches from a SQL database.

=item C<ldap:>

Look up matches from an LDAP directory service.

=item C<perl:>

Let a custom Perl module do the matching.

=back

=head2 Match type

The following match types are supported:

=over

=item C<is>

Exact match which might be modified by a comparator.

=item C<contains>

Substring match which might be modified by a comparator.

=item C<matches>

Glob style matching which might be modified by a comparator.  The
wildcards are '*' (which matches any sequence of characters) 
and '?' (which matches a single character).  A literal '*', 
'?' or '\' might be matched by backslash escaping them.

=item C<nmatches>

Same as C<match>, but list entries starting with '!' are treated as a
negative rule.  During matching each string to be matched is
considered separately.  If it matches at least one of the positive
rules, but none of the negative rules, then it is a match.

As an example, if the nmatches-list contains the following two entries:

   *
   !foo*

Then it will match all strings that do not start with the string
"foo".

=item C<re>

The entries of the list are Perl5 regular expressions which might be
modified by a comparator.  If the list entry starts with '!' then it is
negated as for C<nmatches>.

=item C<domain>

Glob style domain and IP4 address matching.  Matching is always case
insensitive, the setting for comparator is ignored.

If the list entry starts with a decimal number followed by "." or "/",
then it is taken to be an IP4 address with optional mask.  The format
for an entry like this is:

   <inet4>  = <ip> [ "/" <mask> ]
   <ip>     = <digits> [ "." <digits> "." <digits> "." <digits> ]
   <maskip> = <digits> "." <digits> [ "." <digits> "." <digits> ]
   <mask>   = <maskip> | <bits>
   <digits> = integer in the range 0 .. 255
   <bits>   = integer in the range 0 .. 32

Any missing digit sequences in the IP address or mask address are
treated as 0.  This means an IP address or mask specified as
C<192.168> will be treated the same as C<192.168.0.0>.

If the list entry does not start with a number, it is taken as a glob
style string that will be matched against a domain name.  The supported
wildcards are:

   ?   match a single char, but not "."
   *   match a sequence of chars, but not ".".
   **  match a sequence of chars, including "."

Matches are automatically anchored to the end of the string. A leading
"@" can be used to force anchor to the beginning as well.  If the list
item starts with '!' then it is negated as for C<nmatches>.

Some examples follow.

This will match against that exact IP address:

   127.0.0.1

This will match any address in the 10.10.10 network:

   10.10.10.0/24

This will match the same as above:

   10.10.10.0/255.255.255.0

As will this:

   10.10.10/255.255.255

The following will match any hostname in the example.com domain as well as
"example.com" itself.  It will not match "notexample.com" or
"example.com.org".

   example.com

This will match the same hostnames as above, but not "example.com" itself:

   **.example.com


This will match the same hostnames as above:

   .example.com


This will match hosts like "foo.example.com", but not multilevel names
like "foo.bar.example.com":

   *.example.com

This will only match the host "example.com" and not any subdomains:

   @example.com


=item C<mail>

Glob style email address matching.  The wildcards are the same as for
C<domain>, but '@' is prevented from matching on '*' and '?'.  
Also, if the list entry ends with '@' then the match is not
anchored at the end any more.

Some examples:

   example.com

This matches all addresses that end with a domain name of "example.com"
such as "foo@example.com", "foo@foo.example.com".

   @example.com

This matches all addresses with "example.com" as the exact
domain part.

   dev-**@

This matches all addresses that start with the string "dev-".

   foo@perl.*

This matches all addresses with "foo" as the local part and a 
two level domain name with "perl" as the first level, like 
"foo@perl.com" and "foo@perl.org".

   foo@perl.**

This matches all addresses with "foo" as the local part and at 
least a two level domain name with "perl" as the first level, 
like "foo@perl.com" and "foo@perl.domain.com".

=item C<mail-parts>

Match against either a piece of an email address, or the whole
address. When dealing with very large lists or maps, this match
type should operate much faster than the C<mail> match type, and
provides the most commonly-used subset of its functionality.
Globbing is not supported, but whole and partial email addresses
can be matched against. Some examples:

   @example.com

This matches all addresses with "example.com" as the exact
domain part.

   foo@
   
This matches all addresses with "foo" as the exact user part.

   foo@example.com
   
This will only match the exact email address, "foo@example.com".

=back

=head2 Comparator

The following comparators are supported by most match types.  A
comparator modifies how matches are performed.

=over

=item C<i;octet>

There must be exact match between the bytes of the list and the
matched string.

=item C<i;ascii-casemap>

Chars in the ASCII range match even if they differ by case.  This is
the default if no explicit comparator is selected.

=back


=head1 SEE ALSO

L<PureMessage::ListFile>, L<PureMessage::Map>
