package PureMessage::List::Base;

use strict;
use Carp qw(carp croak);

use PureMessage::Version qw($PREFIX);

sub _kind {
    "?";  # override
}

sub Config {
    my $class = shift;
    my $kind = $class->_kind;

    require ActiveState::Config;
    # lists.conf now remains in utf8. 
    my $p = ActiveState::Config->new(strict => 1, is_utf8 => 1);
    $p->typedecl("$kind#*.name" => '#str');
    $p->typedecl("$kind#*.description" => '#str');
    $p->typedecl("$kind#*.precious" => "#bool");
    $p->typedecl("$kind#*.match_type" => '#enum is contains matches nmatches re domain mail mail-parts');
    $p->typedecl("$kind#*.comparator" => '#enum i;octet i;ascii-casemap');
    $p->typedecl("$kind#*.source" => '#str');
    $p->typedecl("$kind#*.items" => '#list #str') unless $kind eq "map";
    $p->typedecl("$kind#*.args.*" => '#str');
    $p->parse_file("$PREFIX/etc/${kind}s.conf", @_);
    return $p;
}

sub _avail_cmp {
    ($a->{precious} ? 0 : 1) <=> ($b->{precious} ? 0 : 1) ||
    lc($a->{name}) cmp lc($b->{name});
}

sub Available {
    my $class = shift;
    my $filter_id = shift;
    my $conf = $class->_config;
    my @avail;
    for my $id (keys %$conf) {
	my $source = $conf->{$id}{source};
	$source = "inline:" unless defined $source;
	unless ($source =~ s/^([a-zA-Z][a-zA-Z0-9.+\-]+:)/lc($1)/e) {
	    $source = "file:$source";
	}
	if ($source =~ s/^(.*file)://) {  # XXX ugly hack
	    my $scheme = $1;
	    my $fclass = $class . "::$scheme";
	    no strict 'refs';
	    unless (@{"$fclass\::ISA"}) {
		eval "require $fclass";
		die $@ if $@;
	    }
	    $source = "$scheme:" . $fclass->_abs_file($source);
	}
	push(@avail, { id => $id,
		       name => $conf->{$id}{name} || $id,
		       description => $conf->{$id}{description} || "",
		       precious => !!$conf->{$id}{precious},
		       source => $source,
		       (exists $conf->{$id}{match_type} ?
			 (match_type => $conf->{$id}{match_type}) : ()),
		       (exists $conf->{$id}{comparator} ?
			 (comparator => $conf->{$id}{comparator}) : ()),
		       (exists $conf->{$id}{args} ?
			 (args => $conf->{$id}{args}) : ()),
		     }) if !defined($filter_id) || $filter_id eq $id;
    }
    return $avail[0] if defined $filter_id;
    return @avail unless wantarray;
    return sort _avail_cmp @avail;
}

BEGIN {
    my %config;
    sub _config {
	my($class, $force_reload) = @_;
	my $kind = $class->_kind;
	delete $config{$kind} if $force_reload;
	return $config{$kind} ||= do {
	    my $p = $class->Config();
	    $p->as_hash->{$kind};
	};
    }
}

sub Setup {
    my($class, %opts) = @_;
    my $kind = $class->_kind;
    my $id = delete $opts{id} || croak("No id specified for $class->Create");

    my $conf = $class->Config(writable => 1, timeout => 5, backup_ext => ".bak");
    croak("A $kind with id '$id' already exists, cannot create")
	if ActiveState::Config::path_node($conf->root, "$kind#$id");

    my $source = delete $opts{source};
    my $touch;
    if ($source) {
      # attempt to have the responsible object initialize the list on creation
      # if that is something that it is able to do
      # TODO this is not exactly right since the init code should live in the
      # Base namespace not the List namespace
      croak("The source format '$source' is unknown") unless $source =~ /^(\w+):(.+)$/;
      my ($type,$init_values) = ($1,$2);
      my $source_class = "${class}::${type}";
      eval {
        eval "require $source_class";
      } ;
      if ($@) {
        croak("The source type '$type' is unknown: $@") unless $source =~ /^(\w+):(.+)$/;
      }
      # try and call the _create method
      # but first.. does this package have such a sub defined?
      my $source_class_symbol = '$' . $source_class . '::{"_create"}';
      my $exists = eval("exists($source_class_symbol)");
      if ($exists) {
        my $init = eval("$source_class->_create(" . '$source,%opts' . ",id => '$id')" );
        if(!$init or $@) {
          croak("Creation of the $type resource failed." . $@ ? "  $@" : "");
        }
      }
      
    } else {
	require PureMessage::List::Base::file;
	my $f = PureMessage::List::Base::file->_abs_file($id);
	croak("The $kind file '$f' already exists, cannot create")
	    if -e $f;
	$touch = $f;
	$source = "file:$id";
    }

    my $node = ActiveState::Config::path_node($conf->root, "$kind#$id",
					      create => "group");

    for (qw(name description match_type comparator)) {
	my $v = delete $opts{$_};
	next unless defined $v;
        ActiveState::Config::path_value($conf->root, "$kind#$id.$_", $v);
    }

    my $args = delete $opts{args};
    if ($args && (ref($args) eq 'HASH')) {
        foreach my $k (keys %{$args}) {
            my $v = delete $args->{$k};
            next unless (defined $v);
            ActiveState::Config::path_value($conf->root, "$kind#$id.args.$k", $v);
        }
    }

    ActiveState::Config::path_value($conf->root, "$kind#$id.source", $source);

    # Resources passes in precious, but we don't care about it
    delete $opts{precious};

    if (%opts && $^W) {
	carp("Unrecognized option '$_' for $class->Create") for sort keys %opts;
    }
    
    if (defined $touch) {
        open(my $f, ">", $touch) || die "Cannot create '$touch': $!";
        close($f);
    }
    
    $conf->save;
    $class->_config(1);
}

sub new {
    my $class = shift;
    my $kind = $class->_kind();

    my %conf;
    if (@_ == 1) {
	my $id = shift;
	my $c = $class->_config->{$id};
	croak("Unknown $kind '$id'") unless $c;
	%conf = (%$c, id => $id);
    }
    else {
	%conf = @_;
    }

    my @args;

    if (my $source = $conf{source}) {
	croak("Not allowed to specify items together with source")
	    if $conf{items};
	my $scheme = ($source =~ s/^([a-zA-Z][a-zA-Z0-9.+\-]+)://) ? $1 : "file";
	$class .= "::" . lc($scheme);
	@args = ($source, %conf);
    }
    elsif ($conf{items}) {
	$class .= "::Items";
	@args = (%conf);
    }
    else {
	croak("Either items or source must be specified for $kind");
    }

    no strict 'refs';
    unless (@{"$class\::ISA"}) {
	eval "require $class";
	die $@ if $@;
    }
    return $class->new(@args);
}

sub id          { shift->{id}   }
sub name        { my $o = shift; $o->{name} || $o->{id} }
sub description { shift->{description} }
sub match_type  { shift->{match_type} }
sub comparator  { shift->{comparator} }
sub source      { shift->{source} }
sub mtime       { shift->{mtime} }

sub data {
    my $self = shift;
    unless (exists $self->{listdata}) {
        my $source = $self->{source};
        my $class = ref($self);
        $class =~ s/^(.+?::.+?)(::.+)$/$1Data/;
        croak "Need source parameter to edit list data" unless defined $source;
        
        my $scheme = ($source =~ s/^([a-zA-Z][a-zA-Z0-9.+\-]+)://) ? $1 : "file";
        $class .= "::" . lc($scheme);
        my @args = ($source, %$self, @_);
            
        no strict 'refs';
        unless (@{"$class\::ISA"}) {
                eval "require $class";
                die $@ if $@;
        }
        $self->{listdata} = $class->new(@args);
    }
    return $self->{listdata};
}

sub refresh {
    # nothing to do by default
    return 0;
}

sub warn {
    my($self, $msg) = @_;
    if (my $id = $self->id) {
	my $kind = $self->_kind;
	$msg = "$kind $id: $msg";
    }
    warn $msg;
}

1;
