package PureMessage::List::Base::ldap;

use strict;
use Net::LDAP;
use Net::LDAP::Constant qw(LDAP_OPERATIONS_ERROR);
use Carp qw(croak);

sub new {
    my($class, $source, %conf) = @_;

    my $args = $conf{args} || croak("No args provided");
    $args->{source} = $source;

    # Default to subtree search.
    $args->{scope} ||= 'sub';

    my $self = \%conf;
    bless $self, $class;

    # Number of times to try reconnecting to the LDAP server on an
    # LDAP search failure.
    $self->{num_retries} = 3;

    # XXX: Undocumented config argument
    if (defined $self->{args}->{retries}) {
        my $self->{num_retries} = $self->{args}->{retries};
        $self->{num_retries} = 0 if $self->{num_retries} < 0;
    }

    return $self;
}

sub DESTROY {
    my $self = shift;
    $self->{ldap}->unbind if ref($self) and $self->{ldap};
}

sub ldap_clear_connection {
    my $self = shift;
    if ($self->{ldap}) {
        eval { $self->{ldap}->unbind() };
        delete $self->{ldap};
    }
}

sub ldap_connect {
    my $self = shift;
    my $args  = $self->{args};
    my $match = $args->{match} || croak("No match statement provided");
    my $port = $args->{port};
    # XXX could optimize this by caching one connection per source
    # but we need to be careful about a memory leak (DESTROY()
    # will never be called in a naive cache implementation)
    my @sources = split /;/, $args->{source};
    my $ldap = Net::LDAP->new(\@sources,
                              ($port ? (port => $port) : ()),
			      version => $args->{version} || 3,
			      async   => $args->{async} || 0,
			      debug   => $args->{debug} || 0,
                             )
                  || die(
                      "Failed to create LDAP connection to (",
                      join(", ", @sources), ")\n"
                  );
    my $m;
    if ($args->{password}) {
	$m = $ldap->bind(dn => $args->{dn},
	                 password => $args->{password},
			 );
    }
    else {
        # anonymous bind
	$m = $ldap->bind;
    }
    $m->code and croak $m->error;

    $self->{ldap} = $ldap;
}

sub ldap_search {
    my ($self, %args) = @_;
    my $msg;
    my $connect_err = 0;
    my $search_err = 0;
    for (;;) {
        while (!$self->{ldap}) {
	    # If there is an error connecting to the LDAP server, this
	    # while loop will try to reconnect, and fail silently until
	    # the final attempt (at which point we croak).
	    eval { $self->ldap_connect() };
	    if ($@) {
		++$connect_err;
		croak($@) if $connect_err > $self->{num_retries};
		next;
	    }
	    last;
        }
	if ($self->{ldap}) {
	    warn "Reconnected to LDAP server.\n" if $search_err;
	}
	else {
            # XXX: Leaving parens out doesn't work: produces compile-time error.
	    croak("Error connecting to LDAP server.\n");
	}
        # Perform the search against the LDAP server.
        $msg = $self->{ldap}->search(%args);
        if ($msg->is_error) {
            ++$search_err;
            $self->ldap_clear_connection();
            if ($search_err > $self->{num_retries}) {
		warn "Error searching LDAP server.\n";
		last;
	    }
            warn "Error searching LDAP server; trying to reconnect...\n";
            next;
        }
        last;
    }
    return $msg;
}

1;
