# PureMessage::List::Items



# the ::retie package is just a stub to allow us to take an already-tie()d
# (or tie-worthy) object and tie it to a(nother?) hash.
#
# so long as the writer of the tie module understands the difference between
# UNTIE and DESTROY, this shouldn't cause any problems. in most cases, our
# code will be passing this an object that isn't tie()d to anything else
# anyway.
#
package PureMessage::List::Items::retie;

use strict;

sub TIEHASH {
    return $_[1];
}

1;

package PureMessage::List::Items;

use strict;

use base qw(PureMessage::List Exporter);

use PureMessage::List::Utils qw(_str);
use Carp;

our $DEBUG = $ENV{PMX_LIST_DEBUG};
our $ff;  # default inet4 mask

our %MatchTypes = (
    is          => 'Exact',
    mail        => 'Email Globs',
    contains    => 'Substring',
    domain      => 'Hostnames and IP Masks',
    re          => 'Regular Expression',
    'mail-parts'  => 'Email Segments',
    matches     => 'Glob',
    nmatches    => 'Negative Glob'
);

our @MatchListDefault = qw(is contains matches domain re);
our @EXPORT_OK = qw(@MatchListDefault %MatchTypes);


sub _generate_matcher {
    my($self, $tie) = @_;

    my $match_type = $self->{match_type} ||= 'is';
    my $ascii_casemap = !$self->{comparator} ||
                    $self->{comparator} eq 'i;ascii-casemap';

    my @items = @{$self->{items}} unless($tie);

    # Generate a sub that will match the set defined by the
    # list based on the given match_type
    my @match_code;
    my @match_code_INIT;

    push(@match_code, "sub {\n");

    if ($ascii_casemap && $match_type ne "re") {
	push(@match_code, "  local \$_;\n");
	push(@match_code, "  while (\@_) {\n");
	push(@match_code, "    my \$orig = \$_[0];\n");
	push(@match_code, "    \$_ = lc(shift);\n");
    }
    else {
	push(@match_code, "  for (\@_) {\n");
	push(@match_code, "    my \$orig = \$_;\n") if $match_type ne "re";
    }

    if ($match_type eq "is" || $match_type eq "mail-parts") {
	push(@match_code, "    return 1 if exists \$match{\$_};\n");
	if ($match_type eq "mail-parts") {
	    push(@match_code, "    my \$at_symbol = index(\$_, '\@');\n");
	    push(@match_code, "    return 0 if(\$at_symbol == -1);\n");
	    push(@match_code, "    return 1 if exists \$match{substr(\$_, \$at_symbol)};\n");
	    push(@match_code, "    return 1 if exists \$match{substr(\$_, 0, \$at_symbol + 1)};\n");
	}

	if ($ascii_casemap) {
	    tr/A-Z/a-z/ for @items;
	}
	
	if ($tie) {
	    push(@match_code_INIT, "    # tie()d to $tie\n")
		if($DEBUG);
	    push(@match_code_INIT, "    my \%match;\n");
	    push(@match_code_INIT, "    tie \%match, 'PureMessage::List::Items::retie', \$tie;\n");
	}
	else {
	    _init_hash(\@match_code_INIT, "match", @items);
	}
    }
    elsif ($tie) {
        croak q{tie()d matchers only supported for "is" and "mail-parts" match types"};
    }
    elsif ($match_type eq "contains") {
	for (@items) {
        my $non_lc = $_;
	    tr/A-Z/a-z/ if $ascii_casemap;
        push(@match_code, "    if ((my \$idx = index(\$_, ",_str($_), ")) >= 0) {\n");
        push(@match_code, "        return wantarray ? (1, substr(\$orig, \$idx, ", length($non_lc), ")) : 1;\n");
        push(@match_code, "    }\n");
	}
    }
    elsif ($match_type eq "matches" || $match_type eq "nmatches") {
	push(@match_code, '    return wantarray ? (1, substr($orig, $-[0], $+[0]-$-[0])) : 1 if (0');
	my @negative;
	for (@items) {
	    my $neg = ($match_type eq "nmatches" && s/^!\s*//) ? 1 : 0;
	    tr/A-Z/a-z/ if $ascii_casemap;
	    $_ = quotemeta($_);
	    s/(?<!\\)\\\?/./g;
	    s/(?<!\\)\\\*/.*/g;
	    s/\\\\\\([*?])/\\$1/g;
	    $_ = "^$_" unless s/^\.\*//;
	    $_ .= "\\z" unless s/\.\*\z//;
	    if ($neg) {
		push(@negative, $_);
	    }
	    else {
		push(@match_code, "\n      || /$_/");
	    }
	}

	if (@negative) {
	    push(@match_code, ")\n    && !(0");
	    for (@negative) {
		push(@match_code, "\n      || /$_/");
	    }
	}
	push(@match_code, ");\n");
    }
    elsif ($match_type eq "re") {
	push(@match_code, '    return wantarray ? (1, substr($_, $-[0], $+[0]-$-[0])) : 1 if (0');
	my @negative;
	for (@items) {
	    my $neg = s/^!\s*// ? 1 : 0;

	    # Need to escape '/', '@' and '$' so that the string can be
	    # interpolated between // to form an RE.  The snag is that we
	    # should not escape these if they are already escaped, i.e. if
	    # there is an odd number of backslashes in front of them.
	    # For '$' we need special care as it is both an anchor and a
	    # prefix for interpolating variables.  We escape it unless
	    # it is followed by '|', ')' or end-of-string.  We do the
	    # matching on the reverse string to work around perl's limitation
	    # that zero-width look-behind only allow fix-width expressions.
	    $_ = reverse $_;
	    s,([/@])(?=(?:\\\\)*(?:[^\\]|\z)),$1\\,g;
	    substr($_, 0, 0) = "|";
	    s,(?<![|\)])\$(?=(?:\\\\)*(?:[^\\]|\z)),\$\\,g;
	    substr($_, 0, 1) = "";
	    $_ = reverse $_;

	    # test that what we ended up with actually is a
	    # valid regular expression.
	    eval qq("a" =~ /$_/);
	    if ($@) {
		$@ =~ s/ at \(eval \d+\) line 1.*//;
		$self->warn($@);
		next;
	    }

	    if ($neg) {
		push(@negative, $_);
	    }
	    else {
		push(@match_code, "\n      || /$_/");
		push(@match_code, "i") if $ascii_casemap;
	    }
	}
	
	if (@negative) {
	    push(@match_code, ")\n    && !(0");
	    for (@negative) {
		push(@match_code, "\n      || /$_/");
		push(@match_code, "i") if $ascii_casemap;
	    }
	}
	push(@match_code, ");\n");
    }
    elsif ($match_type eq "domain") {
	require Socket;
	$ff ||= Socket::inet_aton("255.255.255.255");
	my @inet4;
	my @domain;
	my @domain_neg;
	for (@items) {
	    s/\s+//g;
	    my $neg = s/^!// ? 1 : 0;
	    if (/^\d+(?:\.\d+){0,3}(?:\/|\z)/) {
                my $inet_info = _inet4_mask($_);
                if ($inet_info) {
		   push(@inet4, $inet_info);
		   push(@{$inet4[-1]}, $neg);
                }
	    }
	    else {
		tr/A-Z/a-z/;
		$_ = quotemeta($_);
		s/(?<!\\)\\\?/[^.]/g;
		s/(?<!\\)\\\*/[^.]*/g;
		s/\\\\\\([*?])/\\$1/g;
		$_ .= '\z';
		unless (s/^\\\@/^/) {
		    $_ = "(?:^|\\.)$_" unless /^\\\./;
		}
		s/(?:\[\^\.\]\*){2,}/.*/g;
		if ($neg) {
		    push(@domain_neg, $_);
		}
		else {
		    push(@domain, $_);
		}
	    }
	}

        # $self->{inet4} = \@inet4;
        # $self->{domain} = \@domain;

	push(@match_code, "    if (/^\\d+(?:\\.\\d+){0,3}\\z/) {\n");
	if (@inet4) {
	    push(@match_code, "      my \$addr = Socket::inet_aton(\$_) or return;\n");
	    push(@match_code, "      my \$masked;\n");
	    my $last_addr = "";
	    my $last_mask = "";
	    for (sort {$b->[2] <=> $a->[2] || $a->[1] cmp $b->[1] || $a->[0] cmp $b->[0]} @inet4) {
		my($addr, $mask, $neg) = @$_;
		if ($mask ne $last_mask) {
		    push(@match_code, '      $masked = $addr');
		    push(@match_code, ' & ' . _str($mask, 1)) if $mask ne $ff;
		    push(@match_code, ";\n");
		    $last_mask = $mask;
		}
		my $pos = 1 - $neg;
		push(@match_code, "      return $pos if \$masked eq " .
		                  _str($addr & $mask, 1) . ";\n");
		$last_addr = $addr;
	    }
	}
	else {
	    push(@match_code, "      # no inet4 tests\n");
	}
	push(@match_code, "    }\n");
	if (@domain) {
	    push(@match_code, "    else {\n");
	    push(@match_code, "      return 1 if (0");
	    for (@domain) {
		push(@match_code, "\n        || /$_/");
	    }
	    if (@domain_neg) {
		push(@match_code, ")\n      && !(0");
		for (@domain_neg) {
		    push(@match_code, "\n        || /$_/");
		}
	    }
	    push(@match_code, ");\n");
	    push(@match_code, "    }\n");
	}
    }
    elsif ($match_type eq "mail") {
	push(@match_code, "    return 1 if (0");
	my @negative;
	my @literal_match_pos;
	my @literal_match_neg;
	for (@items) {
	    my $neg = s/^!\s*// ? 1 : 0;
	    tr/A-Z/a-z/;
	    if (/^\**\z/) {
		$_ = '[^@.]';  # match any non-empty email address
	    }
	    elsif (!/\*/ && !/\?/ && /.\@./) {
		if ($neg) {
		    push(@literal_match_neg, $_);
		}
		else {
		    push(@literal_match_pos, $_);
		}
		next;
	    }
	    else {
		$_ = quotemeta($_);
		s/(?<!\\)\\\?/[^@.]/g;
		s/(?<!\\)\\\*/[^@.]*/g;
		s/\\\\\\([*?])/\\$1/g;
		$_ .= '\z' unless /\@\z/;
		if (/\\\@/) {
		    $_ = "^$_" unless /^\\\@/;
		    s/\\\@\z/(?:\\\@|\\z)/;
		}
		else {
		    $_ = "[\@.]$_" unless /^\\/;
		}
		s/(?:\[\^\@\.\]\*){2,}/[^@]*/g;
	    }
	    if ($neg) {
		push(@negative, $_);
	    }
	    else {
		push(@match_code, "\n      || /$_/");
	    }
	}

	if (@literal_match_pos) {
	    push(@match_code, "\n      || exists \$match{\$_}");
	    _init_hash(\@match_code_INIT, "match", @literal_match_pos);
	}

	if (@negative || @literal_match_neg) {
	    push(@match_code, ")\n    && !(0");
	    for (@negative) {
		push(@match_code, "\n      || /$_/");
	    }

	    if (@literal_match_neg) {
		push(@match_code, "\n      || exists \$not_match{\$_}");
		_init_hash(\@match_code_INIT, "not_match", @literal_match_neg);
	    }
	}
	push(@match_code, ");\n");
    }
    else {
	die "Unsupported match_type '$match_type'";
    }

    push(@match_code, "  }\n  return 0;\n}\n");

    # init stuff
    unshift(@match_code, @match_code_INIT);
    my $name = $self->{id} || "type $match_type";

    unshift(@match_code, qq(#line 1 "generated PureMessage::List code for $name"\n));

    if ($DEBUG) {
	my $id = $self->{id} || "?";
	print "-- list '$id' ($match_type) --\n", @match_code, "\n";
    }

    # compile code
    my $matcher = eval join("", @match_code);
    die $@ if $@;

    return $matcher;
}

sub _new_items {
    new(@_);
}

sub new {
    my($class, %conf) = @_;

    my $self = bless \%conf, $class;
    $self->{matcher} = _generate_matcher($self);
    $self;
}

sub items {
    my $self = shift;
    my $old = $self->{items};
    if (@_) {
	$self->{items} = [(@_ == 1 && ref($_[0]) eq "ARRAY") ? @{$_[0]} : @_];
	$self->{matcher} = _generate_matcher($self);
    }
    @$old;
}

sub match {
    my $self = shift;
    $self->{matcher}->(@_);
}

sub matcher {
    shift->{matcher};
}

sub _init_hash {
    my $code = shift;
    my $name = shift;
    push(@$code, "my %$name = (\n");
    for (@_) {
	push(@$code, "  ", _str($_), " => undef,\n");
    }
    push(@$code, ");\n");
}

sub _inet4_mask {
    $_ = shift;
    my($hoststr, $mask) = split("/", $_);
    my $host = _inet4_pack_addr($hoststr);

    unless($host) {
        warn "Can't resolve host '$hoststr'";
        return undef;
    }

    if ($mask) {
	if ($mask =~ /^\s*\d+\s*$/) {
	    my $addr_bits = length($host) * 8;
	    die "Too many mask bits for address"
		if $mask > $addr_bits;
	    $mask = pack("B*", (1 x $mask) . (0 x ($addr_bits - $mask)));
	}
	else {
	    $mask = _inet4_pack_addr($mask) || die "Bad mask '$mask'";
	}
    }
    else {
	$mask = $ff;
    }
    return [ $host, $mask ];
}

sub _inet4_pack_addr {
    my $addr = shift;
    if ($addr =~ /^[\d.]+\z/) {
	# fill in partial IPs used in CIDR notation
	my $dots = $addr =~ tr/.//;
	$addr .= ".0" x (3 - $dots) if $dots < 3;
    }
    return Socket::inet_aton($addr);
}

1;
