package PureMessage::ListData;
use strict;
use Carp;

sub new;
	
*PureMessage::MapData::new = *PureMessage::ListData::new;

return 1;

sub new {
    my $self = shift;
    my $class = ref($self) || $self;
    my $source = shift;
    
    my $ld;
    if ($source =~ /^([^:]+):(.*)$/) {
        my $abs_class = "$class\::$1";
        eval "require $abs_class";
	croak $@ if $@;
        $ld = $abs_class->new($2, @_);
    }
    else {
        croak("Could not parse source argument '$source'");
    }
    $ld
}

sub COMMANDS () {qw(add remove clear delete_source)}

sub parse_diff {
    my $diff = shift;
    return unless defined $diff;
    my ($command, $list, @items) = split " ", $diff;
    die "Unrecognized command $command" unless grep $command eq $_, COMMANDS();
    my $user;
    $list =~ s/#(.*)// and $user = $1;
    return ($command, $list, $user, @items); 
}

sub make_diff {
    my ($command, $list, $key, @items) = @_;
    die "Unrecognized command $command" unless grep $command eq $_, COMMANDS();
    $list .= "#$key" if defined $key and length $key;
    return join " ", $command, $list, @items;
}

__END__

=head1 NAME

PureMessage::ListData - class for list data editing

=head1 SYNOPSIS

 use PureMessage::List;
 my $list = PureMessage::List->new(%opts);
 my $ld = $list->data;

 unless ($ld->is_readonly) {

    $ld->add_items(qw(foo bar baz));
    
    $ld->remove_items(qw(flip flop fly));
    
    $ld->clear;
    
    $ld->commit;
 }

 my @items;
 $ld->enumerate_items(sub {push @items, shift});

=head1 DESCRIPTION

The C<PureMessage::ListData> class gives access to the data source of
a list. For each list source type (file, dbi, etc), there can be a
ListData backend to edit this source type. 

=over 4

=item B<add_items(@items)>

Add items to the list.

=item B<remove_items(@items)>

Remove items from the list. If the list contains duplicates of an
item, all instances are removed. 

=item B<clear>

Clear the list.  All items are removed.

=item B<delete_source>

Delete the source associated with this list. This is useful with
multidimensional lists when you want to delete one of the "keys".
Requires a commit() like other methods. Not recommended for flat file
lists. 

=item B<commit>

Commit changes to the list. This does not prevent more changes using
this object, but it flushes the current changes.

=item B<enumerate_items( sub{ my $item = shift } )>

Call a sub for each item in the list. Calling enumerate_items causes an
implicit commit.

=item B<is_readonly>

Returns true if the list data is read-only, false otherwise.

=back

=head1 EXCEPTIONS

If a list is read-only, only enumerate_items can be used - add_items,
remove_items, clear and commit will croak if called on a read-only
list.

The commit method will croak if it can't commit the data for some
reason. The ListData object will always try to autovivify the list data
source on commit, if it doesn't exist already (if there is new data to
add).

=head1 IMPLEMENTATION-SPECIFIC OPTIONS

=over 4

=item $ld = $list->data(prepend=>1)

The 'file' backend supports a "prepend" option.  This option will cause all
new items to be added to beginning of the list, instead of at the end.  All
other backends will ignore this option.

=item B<commit(timeout => 5)>

The 'file' backend supports a "timeout" option.

For the file backend, the commit() method exclusively locks access to
the file. If another process/thread tries to call commit() during this
time, that process will block waiting for the lock. The default
timeout is 5 seconds. Other timeout delays can be specified with the
timeout option. A timeout of 0 means wait forever. See
L<ActiveState::File::Atomic> for more information. If commit() times
out, it will croak, and the changes will not be flushed. That means it
is possible to catch the exception and try to commit() again. This
method will create a lock file when it is called (e.g. if the list file
is "/etc/foo" the lock will be "/etc/.~foo.lock") that is not
automatically cleaned up. The reason the original file is not locked is
that it is possible to delete the list file in commit() (see the
delete_source method above), and this will break the locking system.

=back

=head1 AUTHOR

ActiveState Corporation

=head1 COPYRIGHT

Copyright (C) 2003, ActiveState Corporation.
All Rights Reserved.

=head1 SEE ALSO

L<PureMessage::List>, L<PureMessage::MultiList>
