package PureMessage::ListData::file;
use strict;
use warnings;

use Carp;
use Errno 'ENOENT';

use PureMessage::ListData::Base;
use base 'PureMessage::ListData::Base';
use PureMessage::List::Utils 'list_absfile';
use File::Path 'mkpath';
use File::Basename;
use PureMessage::ListData;
use PureMessage::_sys qw();
use ActiveState::File::Atomic;
use File::Spec;

sub new {
    my $self = shift;
    my $class = ref($self) || $self;
    my $id = shift;
    my %opts = @_;
    
    my $filename = list_absfile($id);
    my @path = File::Spec->splitpath($filename);   
    my %hash = (
        id => $id,
        filename => $filename,
        tempfile => File::Spec->catpath($path[0], $path[1], ".~$path[2].tmp"),
        lockfile => File::Spec->catpath($path[0], $path[1], ".~$path[2].lock"),
        prepend => $opts{prepend},
        ld_queue => $opts{ld_queue},
    );
    
    my $o = bless \%hash, $class;
    return $o;    
}

sub commit {
    my $self = shift;
    my %opts = @_;
    my %at_opts;
    # 5-second lock by default
    $at_opts{timeout} = exists $opts{timeout} ? $opts{timeout} : 5;
    
    my $affected = 0;
    
    return unless $self->_modified;
    
    my $dir = dirname $self->{filename};
    -d $dir or mkpath $dir or croak "$dir: $!";
    
    # take write lock for this file - this lock is only used by this method.
    my $at = ActiveState::File::Atomic->new($self->{lockfile}, writable => 1, create => 1, %at_opts);
    
    my $to_add = delete $self->{to_add};
    my $to_add_order = delete $self->{to_add_order};
    my $to_remove = delete $self->{to_remove};
    my $clear = delete $self->{clear};
    my $delete_source = delete $self->{delete_source};

    my $num_to_add = scalar keys %$to_add;
    
    if (my $queue = $self->{ld_queue}) {
        if ($clear) {
            $queue->add('clear');
        }
        if ($delete_source) {
            $queue->add('delete_source');
        }
        foreach my $key (keys %$to_remove) {
            $queue->add('remove', undef, undef, $key);
        }
        _to_add({%$to_add}, $to_add_order, sub{$queue->add('add', undef, undef, shift)});
    }
    
    open my $OUT, ">", $self->{tempfile} or croak "$self->{tempfile}: $!";
    if ($self->{prepend}) {
        $affected += _to_add($to_add, $to_add_order, sub {print $OUT shift, "\n"});
    }
    unlink $self->{filename} or die "Couldn't unlink $self->{filename}: $!" if $delete_source and -f $self->{filename};
    if(open my $IN, "<", $self->{filename}) {
        while (<$IN>) {
			# subclasses of ListData-file may want to read multiple items from
			# a single line.
			my @items = $self->parse($_);
			my @keep;
			foreach my $item (@items) {
                if (
	    			   defined $item && length "$item" and
		    	       ($to_remove->{"$item"} or $clear)
			       ) {
                    $affected++;
                } else {
					push(@keep, $item);
				}
			}
			
			if(@keep == @items) {
                print $OUT $_;
			} elsif(@keep) {
				print $OUT join("\n", @keep), "\n";
			}
        }
        close $IN;
    }
    elsif ($! == ENOENT) {
        # if the file doesnt exist, do nothing unless we are adding to it
        unless ($num_to_add) {
            close $OUT;
            unlink $self->{tempfile};
            return $affected
        }
    }
    else {
        croak "$self->{filename}: $!";
    }
    unless ($self->{prepend}) {
        $affected += _to_add($to_add, $to_add_order, sub {print $OUT shift, "\n"});
    }
    close $OUT;
    rename $self->{tempfile}, $self->{filename} 
        or croak "Could not rename '$self->{tempfile}' to '$self->{filename}': $!";
    
    # unlock
    $at->close();
        
    return $affected;
}

sub enumerate_items {
    my $self = shift;
    my $cb = shift;
    $self->commit;
    if (open FH, $self->{filename}) {
	local $_;
        my($x, @x);
        while ($x = <FH>) {
            @x = $self->parse($x)
                or next;
            # parse will return '' when it encounters a comment
            # don't pass those back to the callback
            length($_) && $cb->($_) foreach (@x);
        }
        close FH;
    }
    elsif ($! != ENOENT) {
        croak "$self->{filename}: $!";
    }
}

1;
