package PureMessage::ListFile;

use strict;
use ActiveState::File::Atomic;
use ActiveState::UTF8;

sub new {
    my($class, $file, %opts) = @_;
    my $f = ActiveState::File::Atomic->new($file, %opts);
    my $self = bless {
	file => $f,
	%opts,
    }, $class;

    my @lines;
    while (defined(my $line = $f->readline)) {
	push(@lines, [$self->_parse_line($line)]);
    }
    $self->{lines} = \@lines;

    $self;
}

sub _parse_line {
    my $self = shift;
    local $_ = shift;
    my($entry, $comment);
    if (s/^\s*\"((?:[^\"\\]|\\[\"\\])*)\"//) {
	$entry = $1;
	$entry =~ s/\\(.)/$1/g;
	s/^\s+//;
	s/\s+$//;
	if (length) {
	    s/^\#\s?//;
	    $comment = $_;
	}
    }
    else {
	if (s/(?:^|\s)\#\s?(.*)//) {
	    $comment = $1;
	}
	s/^\s+//;
	s/\s+$//;
	$entry = $_ if length;
    }
    return (ActiveState::UTF8::decode_utf8($entry),
	    ActiveState::UTF8::decode_utf8($comment));
}

sub lines {
    my $self = shift;
    return scalar(@{$self->{lines}});
}

sub _entry {
    $_[1];
}

sub entry {
    my $self = shift;
    my $i = shift;
    my $old = $self->{lines}[$i][0];
    $self->{lines}[$i][0] = $self->_entry(shift) if @_;
    $old;
}

sub entries {
    my($self, $with_index) = @_;
    my $lines = $self->{lines};

    return grep defined($_ && $_->[0]), @$lines unless wantarray;

    my @res;
    for my $i (0 .. @$lines - 1) {
	if (defined(my $e = ($lines->[$i] && $lines->[$i][0]))) {
	    push(@res, $i) if $with_index;
	    push(@res, $e);
	}
    }
    return @res;
}

sub comment {
    my $self = shift;
    my $i = shift;
    my $old = $self->{lines}[$i][1];
    $self->{lines}[$i][1] = shift if @_;
    $old;
}

sub delete {
    my $self = shift;
    for my $i (@_) {
	$self->{lines}[$i] = undef;
    }
}

sub append {
    my $self = shift;
    my %seen;
    for (@{$self->{lines}}) {
	next unless defined($_) && defined($_->[0]) && length($_->[0]);
	$seen{"$_->[0]"}++;
    }

    my $count = 0;
    my @entries;
    while (@_) {
	my($entry, $comment) = splice(@_, 0, 2);
        @entries = $self->_entry($entry);
        shift(@entries) while(@entries && $seen{"$entries[0]"});
        next if(!@entries);
        push(@{$self->{lines}}, [shift(@entries), $comment]);
        $count++;
        while($_ = shift(@entries)) {
            if(!$seen{"$_"}) {
                $count++;
                push(@{$self->{lines}}, [ $_ ]);
            }
        }
    }
    return $count;
}

sub _file {
    my $self = shift;
    my @res;
    for (@{$self->{lines}}) {
	next unless defined;  # deleted
	if (defined($_->[0]) && length("$_->[0]")) {
	    push(@res, $self->_format_entry($_->[0]));
	    push(@res, " ") if defined($_->[1]);
	}
	if (defined $_->[1]) {
	    push(@res, "# $_->[1]");
	}
	push(@res, "\n");
	
    }
    ActiveState::UTF8::encode_utf8(join("", @res));
}

sub _format_entry {
    for ($_[1]) {
	if (/[\s\#\"]/) {
	    my $x = $_;
	    $x =~ s/([\\\"])/\\$1/g;
	    return qq("$x");
	}
	else {
	    return $_;
	}
    }
}

sub commit {
    my $self = shift;
    $self->{file}->commit_string($self->_file);
}

1;

__END__

=head1 NAME

PureMessage::ListFile - Listfile edit interface

=head1 SYNOPSIS

  use PureMessage::ListFile;

  my $list = PureMessage::ListFile->new("blacklist", writable => 1);
  $list->append('foo@bar.com', "Spammers");
  $list->commit;

=head1 DESCRIPTION

The C<PureMessage::ListFile> class can be used to examine or modify
the entries and comments found in a list file.

Methods:

=over

=item $list = PureMessage::ListFile->new( $filename, %opts )

This construct a new ListFile object.  Parameters are the same as for
the ActiveState::File::Atomic constructor.  The constructor will croak
if the file does not exist or other system level errors occour.

=item $list->lines

Returns how many lines there are in the file.

=item $list->entry( $linenum )

=item $list->entry( $linenum, $new_entry )

Returns the value of the entry found at the given line.  If there is
no entry on the line return undef.  With a second argument update the
entry on the given line as well as returning the old value.

Line numbers start at 0.  Negative line numbers count from the end.

=item $list->comment( $linenum )

=item $list->comment( $linenum, $new_comment )

Returns the comment found at the given line.  If there is no comment on
the line return undef.  With a second argument update the
comment on the given line as well as returning the old value.

Line numbers start at 0.  Negative line numbers count from the end.

=item $list->entries

=item $list->entries( $with_index )

Returns all the entries in the file.  In scalar context returns the
number of entries in the file.

If the $with_index argument is given a true value, then index/entry
pairs are returned instead.

=item $list->delete( $linenum, ... )

Deletes the given line from the file.  Subsequent entries and comments
are not shifted up.  The line can be resurrected by assigning a
comment or entry to it.

Line numbers start at 0.  Negative line numbers count from the end.

=item $list->append( $entry )

=item $list->append( $entry => $comment )

Append the entry to the file unless it is already there.  Multiple
entry/comment pairs might be provided.  The return value is the number
of entries added.

=item $list->commit

Commit any changes done.  The file might change even if no explicit
modifications have been made because comments and entry formatting
will be normalized.

=back

=head1 SEE ALSO

L<PureMessage::List>, L<ActiveState::File::Atomic>
