package PureMessage::ListManager;
use strict;
use warnings;
use Carp;
use PureMessage::List;
use PureMessage::MultiList;

sub new {
    my $class = shift;
    my %opts = @_;

    $opts{fetch} ||= [ qw(List MultiList) ];

    my %conf;
    foreach my $i (@{$opts{fetch}}) {
        my $listclass = "PureMessage::" . $i;
        unless($listclass->can('new')) {
            eval qq{use $listclass; 1}
                or croak $@;
            unless($listclass->can('new')) {
                croak "$listclass is not usable by $class";
            }
        }
        unless($listclass->isa('PureMessage::List::Base')) {
            croak "$listclass is not a list-like class";
        }
        $conf{$i} = { _PerlClass => $listclass, $listclass->Config->as_hash };
    }
    
    my %hash = (
        cache => {},
        %opts,
        conf => \%conf
    );
    
    my $self = bless \%hash, $class;
    
    if (exists $opts{preload}) {
        my $queueing = delete($self->{queueing}) || undef;
        my @preload = ref($opts{preload}) ? @{$opts{preload}} : $opts{preload};
        $self->find_list($_, want_ml => 1) foreach (@preload);
        $self->{queueing} = $queueing if(defined($queueing));
    }
    
    return $self;
}

sub find_list {
    my $self = shift;
    my $id = shift;
    my %args = @_;
    my $want_ml = delete($args{want_ml}) || undef;
    
    croak "Parameter 'id' is required" unless defined $id;
    
    my ($list, $list_conf, @extra_args);

    if ($self->{queueing}) {
        require PureMessage::ListData::SyncQueue;
        my $queue = PureMessage::ListData::SyncQueue->new(
            list=>$id, key=>$args{key}, add=>1
        );
        @extra_args = (ld_queue=>$queue);
    }

    # ensure we follow the priority specified in fetch parameter
    unless($list = $self->{cache}{$id}) {
        foreach my $i (@{$self->{fetch}}) {
            if($list_conf = $self->{conf}{$i}{lc($i)}{$id}) {
                $list = $self->{cache}{$id} =
                    $self->{conf}{$i}{_PerlClass}->new(
                        %$list_conf, @extra_args
                    );
                last;
            }
        }
    }

    _croak_unknown_list($id) unless($list);
    
    if($list->can('find_list') && !$want_ml) {
        eval { $list = $list->find_list(%args); };
        if($@) {
            croak "$@ ";
        }
    }

    return $list;
}

sub enumerate_keys {
    my $self = shift;
    my $id = shift;

    my $list = $self->find_list($id, want_ml => 1);
    
    if($list->can('enumerate_keys')) {
        $list->enumerate_keys(@_);
    }
}

sub is_multilist {
    my $self = shift;
    my $id = shift;
    _croak_unknown_list($id) unless $self->list_exists($id);
    foreach (grep(exists $self->{conf}{$_}{lc($_)}{$id}, @{$self->{fetch}})) {
        if($_ eq "MultiList") {
            return 1;
        } else {
            return;
        }
    }
    return;
}

sub list_exists {
    my $self = shift;
    my $id = shift;
    return unless defined $id;
    return grep($self->{conf}{$_}{lc($_)}{$id}, @{$self->{fetch}}) ? 1 : undef;
}

sub clear_cache {
    my $self = shift;
    $self->{cache} = {};
}

sub refresh_lists {
    my $self = shift;
    my $lists = $self->{cache};
    $lists->{$_}->refresh foreach keys %$lists;
}

sub _croak_unknown_list {
    my $id = shift;
    if (defined $id) {
        croak "Unknown list '$id'.  Please check that this list ID is correct.";
    }
    else {
        croak "Undefined list id.";
    }
}

sub patcher {
    my $self = shift;
    return PureMessage::ListManager::Patcher->new($self);
}

1;

package PureMessage::ListManager::Patcher;
use strict;
use PureMessage::ListData;
sub new {
    my $proto = shift;
    my $class = ref ($proto) || $proto;
    my $lm = shift;
    return bless { lm => $lm }, $class;
}

sub patch {
    my $self = shift;
    my $lm = $self->{lm};
    my %lds;
    while(my $diff = shift) {
        my ($command, $list, $key, @items) = PureMessage::ListData::parse_diff($diff);
        my $listspec = defined $key ? "$list#$key" : $list;
        unless ($lds{$listspec}) {
            my $list_obj = $lm->find_list($list, key=>$key, edit_mode=>1);
            $lds{$listspec} = $list_obj->data;
        }
        my $ld = $lds{$listspec};
        if ($command eq 'add') {
            $ld->add_items(@items);
        }
        elsif ($command eq 'remove') {
            $ld->remove_items(@items);
        }
        elsif ($command eq 'clear') {
            $ld->clear;
        }
        elsif ($command eq 'delete_source') {
            $ld->delete_source;
        }
    }
    $lds{$_}->commit foreach keys %lds;
}

sub diff {
    my $self = shift;
    my $lm = $self->{lm};
    
    my $id = shift;
    my $key;
    $id =~ s/#(.*)$// and $key = $1;
    die "Unknown list ID '$id'" unless $lm->list_exists($id);
    my @lists;
    my %listdata;
    if ($lm->is_multilist($id)) {
        if ($key) {
            if (my $list = $lm->find_list($id, key=>$key)) {
                my $listspec = "$id#$key";
                push @lists, $listspec;
                $listdata{$listspec}{ld} = $list->data;
                $listdata{$listspec}{key} = $key;
                $listdata{$listspec}{id} = $id;
            }
        }
        else {
            $lm->enumerate_keys($id, sub {
                my $key = shift;
                if (my $list = $lm->find_list($id, key=>$key)) {
                    my $listspec = "$id#$key";
                    push @lists, $listspec;
                    $listdata{$listspec}{ld} = $list->data;
                    $listdata{$listspec}{key} = $key;
                    $listdata{$listspec}{id} = $id;
                }
            });
        }
    }
    else {
        if (my $list = $lm->find_list($id)) {
            push @lists, $id;
            $listdata{$id}{ld} = $list->data;
            $listdata{$id}{key} = undef;
            $listdata{$id}{id} = $id;
        }
    }
    my @diffs;
    foreach my $list (@lists) {
        push @diffs, PureMessage::ListData::make_diff('clear', $listdata{$list}{id}, $listdata{$list}{key});
        $listdata{$list}{ld}->enumerate_items(sub{
           push @diffs, PureMessage::ListData::make_diff('add', $listdata{$list}{id}, $listdata{$list}{key}, shift); 
        });
    }
    return @diffs;
}

1;
__END__

=head1 NAME

PureMessage::ListManager - class for retrieving lists, wrapper class for
PureMessage::Multilist

=head1 SYNOPSIS

    use PureMessage::ListManager;
    
    my $lm = PureMessage::ListManager->new();
    
    my $list = $lm->find_list('end-user-whitelist', key=>'me@sophos.com');
    
    my @users;
    if ($lm->is_multilist('end-user-whitelist')) {
        $lm->enumerate_keys('end-user-whitelist', sub { push @users, shift });
    }
    
    unless ($lm->list_exists($id)) {
        croak "List '$id' not found!";
    }
    
    $list = $lm->find_list('anti-spam-optouts');
    
    $lm->refresh_lists;
    
    $lm->clear_cache;

=head1 DESCRIPTION

The C<PureMessage::ListManager> is a factory object that builds
PureMessage::List objects.  It also wraps PureMessage::MultiList objects so
that they can be accessed more transparently.

The C<PureMessage::ListManager> object reads the list configuration files once
and uses this data to build list objects.  It also maintains a cache of List
and MultiList objects to reduce redundant work.

=over 4

=item B<new(preload=E<gt>\@list_ids, fetch=E<gt>\@list_types)>

Returns a new PureMessage::ListManager object.

The preload option is a ref to an array of list/multilist IDs.  Lists with
these IDs will be loaded into the cache initially.

The fetch option specifies what types of lists (List, MultiList, Map, etc)
should be loaded, and what order they should be loaded in. If not specified,
then [ List, MultiList ] will be used for the fetching priority.

=item B<find_list($id, %opts)>

Returns a PureMessage::List object (or another of the same list-type)

If the ID belongs to a multilist, then %opts are passed to the multilist's
find_list method and the resulting list is returned.  See
L<PureMessage::MultiList> for more information.

If the ID belongs to a multilist and "want_ml" is specified with a TRUE
value in %opts, then the multilist itself is returned.

If the ID belongs to a list, then the list is simply returned and %opts is
ignored.

=item B<enumerate_keys($id, sub { my $key = shift })>

Call sub for each of the keys/users of a multilist with the given ID.

=item B<list_exists($id)>

Return true if a list/multilist exists with the specified ID.

=item B<is_multilist($id)>

Return true if the list with the specified ID is a multilist.

=item B<refresh_lists()>

Call the refresh method of every list/multilist in the cache.

=item B<clear_cache()>

Clear all objects from the cache.

=back

=head1 EXCEPTIONS

I<find_list()>, I<enumerate_keys()> and I<is_multilist()> will croak if a list/multilist with the specified ID cannot be found.

=head1 AUTHOR

ActiveState Corporation

=head1 COPYRIGHT

Copyright (C) 2003, ActiveState Corporation.
All Rights Reserved.

=head1 SEE ALSO

L<PureMessage::List>, L<PureMessage::MultiList>
