package PureMessage::Logger;

use strict;
use base qw(Exporter);
use Errno;
our @EXPORT_OK = qw(LOG_DEBUG LOG_INFO LOG_NOTICE LOG_WARNING
		    LOG_ERR LOG_ERROR LOG_CRIT LOG_ALERT LOG_EMERG);

my @syslog_names = ('debug', 'info', 'notice', 'warning',
                    'err', 'crit', 'alert', 'emerg');

# log priority constants (borrowed from syslog)
sub LOG_DEBUG   () { 0 }
sub LOG_INFO    () { 1 }
sub LOG_NOTICE  () { 2 }
sub LOG_WARNING () { 3 }
sub LOG_ERROR   () { 4 }
sub LOG_ERR     () { 4 }     # alias, compatible with syslog
sub LOG_CRIT    () { 5 }
sub LOG_ALERT   () { 6 }
sub LOG_EMERG   () { 7 }

use PureMessage::Config qw(%config);
use PureMessage::_sys;

sub new {
    my $class = shift;
    my $self = bless {}, $class;
    $self->openlog(@_);
    $self;
}

sub openlog {
    my($self, %opts) = @_;
    my $ident = delete $opts{ident} || $self->_log_ident;
    my $log_to = delete $opts{log_to};
    if (defined($log_to)) {
	$log_to = "$config{log_dir}/$log_to"
	    unless $log_to =~ m,^/, or $log_to =~ /^syslog:/;
    }
    else {
	$log_to = $config{log_to};
    }

    $log_to =~ s/%%IDENT%%/$ident/g;

    if ($log_to =~ /^syslog:(\S+)/) {
	my $where = $1;

	require Sys::Syslog;
	if ($where eq 'local') {
	    ($^O eq "solaris") ? Sys::Syslog::setlogsock('stream') : Sys::Syslog::setlogsock('unix');
	}
	else {
	    die "Non-local syslog option '$where' not supported, sorry.\n";
	}

	Sys::Syslog::openlog(
	    $ident, # first openlog() in the process "wins" the ident,
		    # because syslog only supports one ident per process. 
	    'pid',  # prepend process id to messages
	    'mail', # part of the "mail" subsystem
	);
    }
    else {
	# this is strictly a sanity check
	open(my $logfh, ">>", $log_to) || die "Can't append to $log_to: $!";
	close $logfh;
    }

    $self->{_logger_logto} = $log_to;
    $self->{_logger_ident} = $ident;
    $self->{_logger_debug_level} = exists($opts{debug_level}) ? $opts{debug_level} : $config{debug_level};
    $self;
}

sub log {
    my($self_or_class, $priority, $msg) = @_;
    $priority ||= "0";
    unless ($priority =~ /^\d+$/) {  # not a number
	no strict 'refs';
	my $pname = "LOG_" . $priority;
	unless (defined &$pname) {
	    $msg = "(Undefined log level '$priority') $msg";
	    $pname = "LOG_ERR";
	}
	$priority = &$pname();
    }
    # see if we have overridden our default debug level or not
    my $current_debug_level = ref($self_or_class) && exists($self_or_class->{_logger_debug_level}) 
                            ? $self_or_class->{_logger_debug_level}
                            : $config{debug_level};

    return if $current_debug_level == 0 && $priority == LOG_DEBUG;
    
    $msg =~ s/^\s+//;
    $msg =~ s/\s+$//;
    $msg =~ s/\s+/ /g;
    my($pkg, $file, $line) = caller;
    $file =~ s,.*/,,;

    my $prefix = $self_or_class->_log_prefix;
    $prefix = "" unless defined $prefix;
    $prefix .= ": " if length $prefix;

    my $logto;
    my $ident;
    if (ref($self_or_class)) {
	$logto = $self_or_class->{_logger_logto};
	unless ($logto) {
	    $self_or_class->openlog;
	    $logto = $self_or_class->{_logger_logto};
	    die "Can't determine log file: $!" unless $logto;
	}
	$ident = $self_or_class->{_logger_ident};
    }
    else {
	my $log = bless {}, $self_or_class;
	$log->openlog;
	$logto = $log->{_logger_logto};
	die "Can't determine log file" unless $logto;
	$ident = $self_or_class->_log_ident;
    }

    if ($logto =~ /^syslog:/) {	
        # to preserve existing behaviour, log everything info and above
        $priority = 3 if (0 < $priority && $priority < 3);
        _do_syslog($ident, $syslog_names[$priority], "$prefix$msg");
    }
    else {
	my $timestamp = _timestamp();

	PureMessage::_sys::append($logto,
	    "$priority $timestamp [" .
	    $self_or_class->_log_pid .
	    ($ident ne "pmx" ? ",$ident" : "") .
	    ($current_debug_level > 3 ? ",$file\:$line" : "") .
	    "] $prefix$msg\n"
	);
    }
}

sub _do_syslog {
    my ($ident, $pri, $msg) = @_;
    return if Sys::Syslog::syslog($pri, '%s', $msg);

    if ($!{EMSGSIZE}) {
	my $len = length $msg;
	_do_syslog($ident, $pri, substr($msg, 0, $len/2));
	_do_syslog($ident, $pri, substr($msg, $len/2));
	return;
    }

    # handle syslogd restarts
    Sys::Syslog::closelog();
    Sys::Syslog::openlog($ident, 'pid', 'mail');
    Sys::Syslog::syslog($pri, '%s', $msg);
}

sub _log_ident {
    "pmx";
}

sub _log_prefix {
    my $self = shift;
    "";
}

sub _log_pid {
    return PureMessage::_sys::tid() if $config{use_threads};
    return $$;
}

sub _timestamp {
    my $utc = $config{log_utc};
    my($sec,$min,$hour,$mday,$mon,$year) = $utc ? gmtime() : localtime();
    sprintf("%04d-%02d-%02dT%02d:%02d:%02d%s",
            $year+1900, $mon+1, $mday, $hour, $min, $sec, ($utc ? "Z" : ""));
}

sub closelog {
    my $self = shift;
    
    if ($self->{_logger_logto} =~ /^syslog:/) {
	Sys::Syslog::closelog();
    }

    delete $self->{_logger_logto};
    delete $self->{_logger_ident};
}

1;

__END__

=head1 NAME

PureMessage::Logger - Interface to write PureMessage log files.

=head1 SYNOPSIS

 use PureMessage::Logger qw(LOG_WARNING);
 $logger = PureMessage::Logger->new( ident => 'prog_name' );
 $logger->log(LOG_WARNING, "Something happened");

 # use a log priority without importing the constant 
 $logger->log("DEBUG", "A debug message");

=head1 DESCRIPTION

The logger appends messages to a local log file. Messages logged with
the DEBUG priority are suppressed unless the debug_level in pmx.conf is
set to 1 or higher.

=head2 Constructor

The constructor accepts the following attributes:

=over

=item ident

This is usually a short string that describes the program that is logging
the messages. This is C<pmx> by default.

=item log_to

This specifies the log file that messages will be appended to. By default
this is the C<pmx_log> as specified in C<pmx.conf>.

=item debug_level

This specifies the current debug_level, overridding the default level as
read from C<pmx.conf>

=back
 
=head2 Logging messages

You append messages to the log file using the C<log> method. The first 
parameter to C<log> is the message priority and the second is the
message string.
Supported log priorities are:

=over

=item LOG_DEBUG

=item LOG_INFO

=item LOG_NOTICE

=item LOG_WARNING

=item LOG_ERROR (can also be spelled LOG_ERR, as in syslog)

=item LOG_CRIT

=item LOG_ALERT

=item LOG_EMERG

=back

These constants are not exported by default. If you don't want to import
these constants you can use a string that contains the log priority name
without the leading C<LOG_>. For example, rather than importing the 
C<LOG_DEBUG> constant, you can simply write the string C<DEBUG> for
the log priority.

=head1 LOG FILE FORMAT
 
The log file written to by PureMessage::Logger contains a sequence of
log entries, one per line.  The format typically looks like this:

  2 2002-02-12T05:06:10Z [16838:3075] g1C569gA016837: accepted

The first field is a number that indicate the log priority level.  The
levels are the same as those found for syslog:

  0 debug
  1 info
  2 notice
  3 warning
  4 error

The next field is a timestamp in ISO 8601 format.  A trailing "Z"
means UTC time.

The next field contains the process id of the process.  If
debug mode is on, then it might also contain a module line and a line
number indicating where the log entry was triggered from.

The rest is of a log entry is free text.  For log messages generated
during the processing of a specific message, then the free text part
will be prefixed by a message queue id as shown in this example.

=head1 SEE ALSO

L<pmx-config>

=head1 COPYRIGHT

Copyright (c) 2003 ActiveState. All rights reserved.

