package PureMessage::Logger::Tee;
use strict;
use base 'PureMessage::Logger';
use IO::Wrap;

# re-export constants so people don't have to include 
# both PM::Logger and PM::Logger::Tee
my @constants = qw(LOG_DEBUG LOG_INFO LOG_NOTICE LOG_WARNING LOG_ERROR LOG_ERR LOG_CRIT LOG_ALERT LOG_EMERG);
PureMessage::Logger->import(@constants);
our @EXPORT_OK = @constants;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my %opts = @_;
    my $tee = delete $opts{tee};
    my $debug = delete $opts{debug};
    my $logger = delete $opts{logger};
    
    my $self = bless {}, $class;
    
    if ($logger) {
        $self->{_tee_logger} = $logger;
    }
    else {
        $self->SUPER::openlog(%opts);
    }
    if ($tee) {
        if (ref $tee eq 'CODE') {
            $self->{_tee_cb} = $tee;
        }
        else {
            my $tee = wraphandle($tee);
            $self->{_tee_cb} = sub {
                my $pri = shift;
                my $msg = shift;
                $msg .= "\n" unless $msg =~ /\n\z/;
                $tee->print($msg)
            };
        }
    }
    $self->{_tee_debug} = $debug;
    $self;
}

sub openlog {
    my $self = shift;
    delete $self->{_tee_logger};
    $self->SUPER::openlog(@_);
}

sub log {
    my $self = shift;
    my $pri = $_[0];
    
    if (($pri =~ /^\d$/ and $pri == LOG_DEBUG()) or ($pri eq 'DEBUG')) {
        $self->{_tee_cb}->(@_) if $self->{_tee_debug};
    }
    else {
        $self->{_tee_cb}->(@_);
    }

    if ($self->{_tee_logger}) {
        $self->{_tee_logger}->log(@_);
    }
    else {
        $self->SUPER::log(@_);
    }
}

1;
__END__

=head1 NAME

PureMessage::Logger::Tee

=head1 SYNOPSIS

use PureMessage::Logger::Tee qw(LOG_WARNING)

my $logger = PureMessage::Logger::Tee->new(
    logger => $some_other_logger,
    tee => STDOUT,
    debug => 1,
);

=head1 DESCRIPTION

PureMessage::Logger::Tee extends PureMessage::Log with the ability
to tee the log output to some other place.

=over 4

=item B<new(%opts)>

Create a new logger.  The following options are supported:

=over 4

=item B<logger => $logger>

The object should log to this other log object.  If this option isn't present, 
a new PureMessage::Logger object is created.  The $logger can be anything with
a log() method.

=item B<tee>

Something to tee the log output to.  Can be a codref, or something that can be
printed to (it is wrapped with IO::Wrap internally).

If it is a coderef, your callback will get all the arguments passed to the log()
command.  If it is a filehandle-like thingy, it will get all but the first 
argument (the priority), and a newline will be appended to the log entry if
there isn't one already.

=item B<debug>

Set to non-zero if you want to tee DEBUG-level messages.  Otherwise you will not
see DEBUG output.

=item B<logto> and B<ident>

These are passed to the new PureMessage::Logger object (ignored if logger is specified).

=back

=item B<log($pri, $msg)>

Same as PureMessage::Logger.

=item B<openlog(%opts)>

Same as PureMessage::Logger.  If this method is called, the logger will start logging
to the new location and the current logger object, if there is one, will be deleted.

=back

=head1 AUTHOR

Sophos PLC

=head1 COPYRIGHT

Copyright (C) 2000-2009 Sophos Group
All Rights Reserved.

=head1 SEE ALSO

L<PureMessage::Logger>, L<IO::Wrap>
