package PureMessage::Map;

use strict;
use base 'PureMessage::List::Base';

sub _kind { "map" }

sub transform {
    my($self, @words) = @_;
    return wantarray ? $words[0] : @words;
}

sub transformer {
    my $self = shift;
    return sub {
	$self->transform(@_);
    }
}

1;

=head1 NAME

PureMessage::Map - Interface to mapping objects

=head1 SYNOPSIS

  use PureMessage::Map;

  my $map = PureMessage::Map->new("notifications");
  for ($map->transform('foo@bar.com')) {
      #...
  }

=head1 DESCRIPTION

C<PureMessage::Map> objects transform strings into other strings.
Their interface is identical to those of C<PureMessage::List> objects,
but instead of match/matcher methods they have transform/transformer
methods.

The following methods that are different or new compared to
C<PureMessage::List>:

=over

=item $map = PureMessage::Map->new( $mapid )

=item $map = PureMessage::Map->new( %opts )

The object constructor has basically the same behaviour as one of
C<PureMessage::List>, but the format of an 'items' entry is different;
for a map it is an array of array references, where the first element
of each inner array describe what to match and the rest how to
transform if the match is found.

=item @strings = $map->transform( $string, ... )

This will transform the arguments strings into 0 or more result
strings.

In scalar context only the first of the results are returned (or
C<undef> if there are no results).

=item $func = $map->transformer

This return a function that works like the transform method above.

=back

The following methods are available with identical behaviour as for
C<PureMessage::List>:

    Config
    Available
    id
    name
    description
    match_type
    comparator
    source
    refresh

=head1 BUGS

Only C<match_type> "is", "mail", and "mail-parts" are currently implemented.

=head1 SEE ALSO

L<PureMessage::MapFile>, L<PureMessage::List>
