#!perl

package PureMessage::MapData::Entry;

use strict;
use warnings;
use Carp qw(croak);
use PureMessage::MapData::Entry::Item;
use ActiveState::UTF8 qw(decode_utf8);

use overload (
    '""'    => \&as_string,
    'eq'    => \&entry_eq,
    'ne'    => \&entry_ne,
	'cmp'	=> \&entry_cmp,
    '@{}'   => \&as_array
);

return 1;

# functions

sub parse {
    my($class, $line, %opts) = @_;
    local $_ = defined($line) ? $line : '';
    return() if /^\s*\#/;
    s/^\s+//;
    s/\s+\z//;
    return() unless length;

    my @items;
    my @map;
    my $colon;
    while (length) {
        if (s/^\"//) {
            # quoted value
            s/^((?:[^\"\\]|\\[\"\\])*?)\"\s*// ||
                croak("Illegal quoted value");
            my $v = $1;
            $v =~ s/\\(.)/$1/g;
            push(@map, $v);
        } elsif (s/^:\s*//) {
            croak("Multiple colons found")
                if defined $colon;
            croak("Colon can not be first")
                unless @map;
            $colon = @map;
        } elsif (/^\#/) {
            last;
        } elsif (s/^([^\s:\#]+)\s*//) {
            push(@map, $1);
        } else {
            die "Assert: Bad line '$_'";
        }
    }

    my @left;
    
    if ($colon && $colon > 1) {
        @left = splice(@map, 0, $colon);
    }
    else {
        @left = shift(@map);
    }
    return($class->new([@left], [@map]));
}

# constructors

sub new {
    my $class = shift;
    my($left, $right, %opts) = @_;
    if(ref($class)) {
        %opts = %$class;
        $class = ref($class);
    }
    
    $opts{ordered}||=0;
    
    # split "multiple left hand" entries into an array of
    # single-keyed values
    if(ref($left) eq "ARRAY") {
        if($#$left == $[) {
            $left = $left->[$[];
        } elsif(wantarray) {
            return(map { $class->new($_, $right) } @$left);
        } else {
            croak "Multiple source entry initialized in scalar context";
        }
    }

    my $self = bless {
        left => 0,
        right => [],
        %opts
    }, $class;
    
    $self->left($left);
    $self->right(ref($right) eq "ARRAY" ? (@$right) : ($right));
    
    $self;
}

# methods

sub itemize {
    my $self = shift;
    my $item = shift;
    if(UNIVERSAL::isa($item, "PureMessage::MapData::Entry::Item")) {
        return $item;
    } else {
        return PureMessage::MapData::Entry::Item->new($item);
    }
}

sub left {
    my $self = shift;

    croak "attempt to assign multiple values to left side of map"
        if(@_ > 1);

    my $value = $self->{left};
    
    if(@_) {
        $self->{left} = $self->itemize($_[0]);
    }

    return $value;
}

sub right {
    my $self = shift;
    my(@value) = (@{$self->{right}});

    if(@_) {
        my(@new_value) = map { $self->itemize($_); } @_;
        $self->{right} = \@new_value;
    }

    return(@value);
}

sub as_string {
    my $self = shift;
    join(
        ": ", $self->left->as_string,
        join(" ", map { $_->as_string } $self->right
    ));
}

sub as_array {
    my $self = shift;
    [$self->left, $self->right];
}

sub _right_sorted {
    my $self = shift;
    return $self->{ordered} ? ($self->right) : sort { $a cmp $b } ($self->right);
}

sub entry_cmp {
	my($aa, $bb) = @_;
	if(ref($bb) && ref($bb) eq ref($aa)) {
		return $aa->left cmp $bb->left;
	} else {
		return $aa->left cmp "$bb";
	}
	
}

sub entry_eq {
    my($aa, $bb) = @_;
    if(ref($bb) && ref($bb) eq ref($aa)) {
        return 0 unless($aa->left eq $bb->left);
        my(@aa) = $aa->_right_sorted;
        my(@bb) = $bb->_right_sorted;
        return 0 unless(scalar(@aa) == scalar(@bb));
        # only make a copy of the array if we have to sort it.
        # (of course in most cases we'll have to sort it anyway...)
        return 0 if grep($aa[$_] ne $bb[$_], $[ .. $#aa);
        return 1;
    } elsif(defined($bb)) {
        return $aa->entry_eq($aa->parse($bb));
    } else {
        return 0;
    }
}

sub entry_ne {
    return ! entry_eq(@_);
}

=pod

=head1 NAME

PureMessage::MapData::Entry - methods for one "row" or "entry" of a Map.

=head1 SYNOPSIS

$entry = PureMessage::MapData::Entry->parse("joe@work.com: joe@home.com joe@vacation.com");

$entry = PureMessage::MapData::Entry->new('joe@work.com' => [ 'joe@home.com', 'joe@vacation.com' ]);

$entry = PureMessage::MapData::Entry->new('routine' => [ qw(wake_up shower clothes breakfast) ], ordered => 1);

@entries = PureMessage::MapData::Entry->parse("sales@work.com support@work.com: joe@work.com");

if($entry1 eq $entry2) { ... }

print "$entry1\n";

if(grep($_ eq 'joe@home.com', $entry->right)) {
    print "joe never checks his email anyway.\n";
}

$entry->left('bob@work.com');

$entry->right('joe@work.com', 'joe@home.com');

my $item = $entry->itemize('jane@work.com');

$sth->execute($instance_id, $entry->left, join(" ", $entry->right));

=head1 DESCRIPTION

PureMessage::MapData::Entry objects represent a single entry in a PureMessage
Map. It provides a facility to import and export entries to/from mapfiles,
compare entries, and manipulate their contents. Both ordered and unordered
entries are supported.

=head1 CONSTRUCTORS

=over

=item parse($string[, ordered => 1 | 0])

parse() takes a string representation of the map entry, parses it, and
returns a PureMessage::MapData::Entry object.

=item new(\@left | $left, \@right | $right[, ordered => 1 | 0])

new() takes the "left" and "right" sides (also known as "keys" and "values")
of the map entry, and returns a PureMessage::MapData::Entry object. The left
and right parameters may be specified as either a string, or array reference.
If the "left" parameter is specified as an array reference, the constructor
will multiplex: one object will be returned for every "left" value specified,
each containing the same "right" values.

An exception will be raised if new() is given multiple "left" values in
scalar context.

=back

=head1 METHODS

=over

=item left([$new])

The "left" method returns the current "left" value, and optionally sets
it to a new value as well. $new may be an unescaped scalar or a
PureMessage::MapData::Entry::Item object.

=item right([@new])

The "right" method returns all of the current "right" values, and optionally
replaces the existing list with a new one. Each argument specified may be an
unescaped scalar or a PureMessage::MapData::Entry::Item object.

=item itemize($value)

The "itemize" method checks if $value is already a member of the
PureMessage::MapData::Entry::Item class. If it is, $value is returned.
If not, then a new PureMessage::MapData::Entry::Item is created with that
value (assumed to be an unescaped scalar) and that is returned instead.

=back

=head1 OPERATORS

=over 

=item "$entry"

When used as part of a string, a PureMessage::MapData::Entry object
will return a properly escaped scalar representation of itself, suitable
for writing to a mapfile.

=item $entry1 eq $entry2 / $entry1 ne $entry2

PureMessage::MapData::Entry objects can be compared with eachother, or with
strings.

When compared with eachother, objects are considered equivalent if their
"left" values are equal, and their "right" values are equal.

The "ordered" argument used during initialization is significant: by default,
the actual order of the "right" values is not significant; [1, 2, 3] is
considered the same as [3, 2, 1]. If either object was initialized with
"ordered" set to TRUE, then both objects' "right" values must both be equal
and in the exact same order in order for them to be considered equivalent.

When compared with a string, the string is parsed as a MapData::Entry object
and that is compared instead.

=item $entry1 cmp $entry2

When PureMessage::MapData::Entry objects are compared to see which is
greater or less-than the other, the value of their "left" arguments are
compared. This is useful for sorting lists of PureMessage::MapData::Entry
objects.

=back

=head1 SEE ALSO

L<PureMessage::MapData::Entry::Item>

=cut
