package PureMessage::Mbox::Parser;
use strict;
use Carp;

my @EVENTS = qw(message_start message_data message_end);

sub new {
    my $class = shift;
    my %opts = @_;
    my $self = bless {}, $class;
    $self->{"_mboxp_event_$_"} = sub {} foreach @EVENTS;
    $self->_reset();
    $self;
}

sub handler {
   my $self = shift;
   my $event = lc shift;
   my $cb = shift;

   croak "'$event' is not a valid event" unless grep $_ eq $event, @EVENTS; 
   $self->{"_mboxp_event_$event"} = $cb;
}

sub parse {
    my $self = shift;
    return unless defined $_[0];
    $self->_parse(exists $self->{_mboxp_partial_line}
                  ? delete($self->{_mboxp_partial_line}) . $_[0]
                  : $_[0]);
}

sub eof {
    my $self = shift;
    # simulate a newline at the end of the buffer, if there isn't one
    $self->_parse(delete($self->{_mboxp_partial_line}) . "\n")
        if exists $self->{_mboxp_partial_line} and $self->{_mboxp_partial_line} !~ /\A\r?\Z/;
    $self->_do_end();
}

sub strip_quoting {
    my $self = shift;
    my $old = $self->{_mboxp_strip_quoting};
    if (@_) {
        $self->{_mboxp_strip_quoting} = shift;
    }
    $old;
}

sub _parse {
    my $self = shift;
    my $data = shift;
    
    return unless length $data;
    # capture From_ line at beginning of buffer
    if ($data =~ s/\A(From .*\n)//){
        $self->_do_from($1);
    }
    while (length $data) {
        # capture until we find a From_ line
        if ($data =~ s/\A(.*?\n)(From .*?\n)//s){
            $self->_do_data($1);
            $self->_do_from($2);
        }
        else {
            # capture all lines in buffer and save partial line at the end
            if ($data =~ s/\A(.*\n)(.*)\z//s) {
                $self->_do_data($1);
                $self->{_mboxp_partial_line} .= $2;
            }
            # there is only one partial line
            else {
                $self->{_mboxp_partial_line} = $data;
                $data = "";
            }
        }
    }
}

sub _do_from {
    my $self = shift;
    my $fdata = shift;
    $self->_do_end();
    my ($envsender, $date, $moreinfo) = ('','','');
    if ($fdata =~ s/\AFrom ([^ \r\n]+)//) {
        $envsender = $1;
        if ($fdata =~ s/\A ([^\r\n]{1,24})//) {
            $date = $1;
            if ($fdata =~ s/\A (.*)\Z//) {
                $moreinfo = $1
            }
        }
    }
    $self->{_mboxp_started} = 1;
    $self->{_mboxp_event_message_start}->($envsender, $date, $moreinfo);
}

sub _do_end {
    my $self = shift;
    if ($self->{_mboxp_started}) {
        $self->{_mboxp_event_message_end}->();
    }
    $self->_reset();
}

sub _do_data {
    my $self = shift;
    my $data = shift;
    return unless $self->{_mboxp_started};
    $data =~ s/(\r?\n)\z// and $self->{_mboxp_partial_line} .= $1;
    $data =~ s/\n(>*)>From /\n$1From /g if $self->{_mboxp_strip_quoting};
    $self->{_mboxp_event_message_data}->($data);
}

sub _reset {
    my $self = shift;
    delete $self->{_mboxp_partial_line};
    $self->{_mboxp_started} = 0;
}

__END__

=head1 NAME

PureMessage::Mbox::Parser

=head1 SYNOPSIS

    my $parser = PureMessage::Mbox::Parser->new;
    
    $parser->handler(message_start => \&handle_start);
    $parser->handler(message_data => \&handle_data);
    $parser->handler(message_end => \&handle_end);
    
    $parser->parse($somedata);
    
    $parser->eof;

=head1 DESCRIPTION

PureMessage::Mbox::Parser parses mbox files, extracting "From " line
metadata and RFC2822 message data.  It follows the guidelines in:

http://www.qmail.org/man/man5/mbox.html

except that stripping ">From " quoting is disabled by default, since
this quoting is broken in many MTAs. This feature can be enabled with
the strip_quoting() method.

=over 4

=item B<new()>

Returns a new parser object.

=item B<handler(message_event=>\&handler)>

Add a handler for each parser event.  Valid parser events are:

=over 4

=item B<message_start>

Fired when a "From " line is hit.  The handler should resemble the
following:

    sub handler {
        my ($envsender, $date, $moreinfo) = @_;
        # do stuff
    }

These fields default to empty strings if they are not present in the
"From " line.  Also the parser does not guarantee that they contain
valid data, so validation may be necessary.

=item B<message_data>

Fired when some message data is available.

The handler should look like:

    sub handler {
         my $data = shift;
    }

=item B<message_end>

Fired when the end of a message is reached.

The handler should look like:

    sub handler {
    }

=back

=item B<eof()>

Tell the parser that there is no more data.  This flushes any remaining
data and resets the parser.

=item B<strip_quoting()>

Get/set the 'strip_quoting' option.  If enabled (non-zero), the parser
will strip ">" characters from lines that begin with ">From ", ">>From
", etc.  See the caveat above.

=back

=head1 AUTHOR

Sophos PLC

=head1 COPYRIGHT

Copyright (C) 2000-2009 Sophos Group
All Rights Reserved.

=head1 SEE ALSO

L<PureMessage>
