package PureMessage::Mbox::Writer;
use strict;
use Carp;
use IO::Wrap qw(wraphandle);

sub new {
    my $class = shift;
    my $fh = shift || croak "Filehandle parameter required";
    my %opts = @_;
    $opts{eol} = "\n" unless defined $opts{eol};
    $opts{block_size} ||= 4096;
    $opts{fh_out} = wraphandle($fh);
    return bless \%opts, $class;
}

sub from {
    my $self = shift;
    my %opts = @_;
    
    my $fromline = "From ";
    $fromline .= defined $opts{env_sender} ? "$opts{env_sender} " : 
        "nobody\@localhost.localdomain ";
    $fromline .= defined $opts{date} ? $opts{date} : scalar(localtime());   
    $fromline .= defined $opts{more_info} ? " $opts{more_info}" : "";    
    $fromline .= $self->{eol};
    $self->{fh_out}->print($fromline);
}

sub data {
    return shift->{fh_out}->print(_quote_from($_[0])); 
}

sub data_fh {
    my $self = shift;
    my $fh_in = shift || croak "Parameter required";
    my $fh_out = $self->{fh_out};
    $fh_in = wraphandle($fh_in);
    my ($block_size, $buf) = $self->{block_size};
    while($fh_in->read($buf, $block_size)) {
        return 0 unless $fh_out->print(_quote_from($buf));
    }
    return 1;
}

sub eom {
    my $self = shift;
    return $self->{fh_out}->print($self->{eol});
}

sub _quote_from {
    (my $quoted = shift) =~ s#\n(>*)From #\n$1>From #g;
    return $quoted;
}

1;

__END__

=head1 NAME

PureMessage::Mbox::Writer

=head1 SYNOPSIS

use PureMessage::Mbox::Writer;

open MBOX, "~/my_mailbox";
my $mbox = PureMessage::Mbox::Writer(\*MBOX);
$mbox->from(env_sender=>'me@here.com');
$mbox->data(<<EOM);
Subject: My Message

Here it is!

EOM
$mbox->eom();
close MBOX;


=head1 DESCRIPTION

Writes messages to a stream in mbox format.  The following methods are
used to write to the mbox stream.  There are no restrictions on what
methods need to be called when, but the recommended order is:

 $mbox->from(%opts);
 $mbox->data($data);
 $mbox->data($data);
 .
 .
 .
 $mbox->data($data);
 $mbox->eom();

If you know that your data is already in mbox format, you should not
need to call from() or eom().

=over 4

=item B<new($fh, %opts)>

Parameters:

=over 4

=item B<$fh>

Filehandle to write mbox data to.  Can be blessed ref or a globref.

=item B<opts{eol}>

Line terminator to use for the From_ line and end-of-message.  Defaults
to "\n".

=item B<opts{block_size}>

The buffer size in bytes for stream I/O.  Defaults to 4096.

=back

=item B<from(%opts)>

Write a From_ line.

Parameters:

=over 4

=item B<$opts{env_sender}>

The envelope sender's email address.  Defaults to
"$ENV{USER}@localhost.localdomain" if $ENV{USER} is defined, otherwise
"nobody@localhost.localdomain" is used.

=item B<$opts{date}>

A asctime()-formatted date string.  Defaults to now.

=item B<$opts{more_info}>

Additional info to add to the end of the From_ line.  Defaults to empty
string.

=back

=item B<data($str)>

Write message data from a string.

=over 4

=item B<data_fh($fh)>

Write message data from a filehandle (a blessed ref that can read() or a
globref).  The block_size option can be used to specify how much data to
read at a time (see new())

=back

=item B<eom()>

Writes a blank line to terminate the message.

=back

=head1 AUTHOR

Sophos PLC

=head1 COPYRIGHT

Copyright (C) 2000-2009 Sophos Group
All Rights Reserved.

=head1 SEE ALSO


