package PureMessage::Message::Sanitize;
use ActiveState::MIME::Parser;
use Data::Dump qw(dump);
use strict;
use warnings;
use HTML::Parser;
use HTML::Entities;
use ActiveState::I18N::UTF8 qw(utf8_encode);
use PureMessage::UTF8::Config qw(%utf8_config);
our $DEBUG = $ENV{DEBUG_SANITIZE};

=head1 NAME

PureMessage::Message::Sanitize - Prepare Email Messages For Safe Viewing

=head1 SYNOPSIS

  use PureMessage::Message::Sanitize;
  use PureMessage::MessageFile;
  
  my $message = PureMessage::MessageFile($message_file);

  my $user_visible = PureMessage::Message::Sanitize::get_preferred_message_part($message_file);


=head1 DESCRIPTION

This package helps with parsing and sanitising email messages for web 
viewing. 

=item get_preferred_message_part($msg_text || $pm_annotedMessageObject[,$max_chars])

This function accepts either text or a C<PureMessage::AnnotedMessage>
object and returns the text of the attachment a users email client is likely
to choose to display.  Returns a hash reference where the content-type is
the key, and part content the value.

If C<$max_chars> is supplied then only a max of $max_chars will be collected. Note
that the memory usage will 3 x this value and that parsing is stopped as soon
as that limit is exceeded, disregarding line breaks etc.  There is a hardcoded
limit of 100k characters.

=cut

sub get_preferred_message_part { 
  my $input = shift;
  my $max = shift || 1024 * 100; # 100 k seems reasonable
  my @parts = ('text/html','text/plain','multipart/alternative'); # parts to collect
  my %found;
  @found{@parts} = ();

  my $collect = 0;
  my $len = 0; # for debug
  my $p = ActiveState::MIME::Parser->new;
  
  my $part_start = sub {
        my ($p, $hdrs, $ct, $ct_param, $te) = @_;
        if (exists($found{$ct}) and !defined($found{$ct})) {
           $collect = $ct;           
           $found{$collect}{charset} = $ct_param->{charset};
        }
        
        if ($DEBUG) {
            print "part_start: ".$p->part_id." ct=",dump($ct)," te=",dump($te);
            for (keys %$ct_param) {
                print " $_=",dump($ct_param->{$_});
            }
            $len += length $hdrs;
            print "\n";
            print "COLLECT: $collect\n";
        }
        
  };

  my $part_end = sub {
        my $p = shift;
        $collect = 0;
        
        print "part_end: ".$p->part_id."\n" if $DEBUG;
  };

  my $data = sub {
        my ($p, $data) = @_;
        if ($collect) {
	  $found{$collect}{data} = "" unless defined $found{$collect}{data};
          $found{$collect}{data} .= $data;
          $collect = 0 if length($found{$collect}{data}) >= $max; # we can have more than $max chars already
        }
        if ($DEBUG) {
            $len += length $data;
            print "data ".$p->part_id.": length=", length $data;
            print ": ", dump($data) if $DEBUG > 10;
            print "\n";
        }
  };

  # we want the mime preamble if there aren't even text/plain parts for that we
  # need the $data_raw content.
  my $data_raw = sub {
        my ($p, $data) = @_;
        return unless $collect eq "multipart/alternative"; # we have to special case this :(
        if ($collect) {
	  $found{$collect}{data} = "" unless defined $found{$collect}{data};
          $found{$collect}{data} .= $data;
          $collect = 0 if length($found{$collect}{data}) >= $max; # we can have more than $max chars already
        }
       
        if ($DEBUG) {
            $len += length $data;
            print "data ".$p->part_id.": length=", length $data;
            print ": ", dump($data) if $DEBUG > 10;
            print "\n";
        }
  };


  $p->handler("part_start" => $part_start);
  $p->handler("part_end"   => $part_end);
  $p->handler("data_decoded" => $data);
  $p->handler("data_raw" => $data_raw);
  # this gives me an error now and it shouldn't
  # $p->decode_te(1); # remove transer encoding
  $p->descend_multipart(1);
  #$p->descend_sub_message(1);
  
  if (UNIVERSAL::isa($input, 'PureMessage::AnnotatedMessage')) {
    # then use the read interface
    $input->seek(0);
    while($input->read(my $buf,2048)) {
      $p->parse($buf);
    }
  } else {
    $p->parse($input);
  }
  
  $p->eof;
  foreach my $results ( @parts ) {
     # return the first part we have that has content
     return { $results => $found{$results} } 
       if defined($found{$results}{data}) && length($found{$results}{data}) > 1; 
  }
  return; # or return undef
}

=item clean_message($pm_annotedMessageObject[,$max_chars])

This function accepts a C<PureMessage::AnnotedMessage> object and returns 
the text of the attachment a users email client is likely to choose to 
display escaped and safe for display in a browser.

=cut

sub clean_message {
  my $pma = shift;
  my $max = shift || 1024 * 100; # 100k is plenty
  my $result = get_preferred_message_part($pma,$max);
  return "" unless $result;
  my ($type,$part) = %$result;
  my $text = $part->{data};
  # Unfortunately the mime parser sets the content charset as 
  # US-ASCII if it does not exist, but we want to guess the
  # charset in this case.  So, for now, just undefine the charset if
  # it is US-ASCII (we used to guess all the time anyway).
  $part->{charset} = undef if $part->{charset} eq 'US-ASCII';
  utf8_encode($text, $part->{charset}, \%utf8_config);
  return clean_html($text);
}


=item restrict_charset($text) 

Removes all control characters from the passed stream and returns the
result.

=cut

sub restrict_charset {
  my $text = shift;
  ## ESC (0x1b) is the encoding-prefix of ISO-2022-JP
  ## follow SkyeC's decision to add it
  # XXX 0x88-0xa0 is the valid range of big5-hkscs
  $text =~ s/[^\x09\x0a\x0d\x1b\x20-\x7e\x88-\xff]/?/g;
  return $text;
}

=pod

=item clean_html($html_text)

This function parses a string of potentially unsafe HTML, santizes it, and returns
a string of sane HTML.

Many of HTML's tags will be dropped.  This is normal behaviour.

=cut

my %ALLOW_TAGS;
@ALLOW_TAGS{qw(br p i b)} = ();

sub clean_html {
    my $html = shift @_;

    my $string = "";

    my $_text = sub {
	my ($text) = @_;
	  $string .= $text;
    };

    my $_start_tag = sub {
	my ($tagname, $dtext, $attr) = @_;

	if (exists $ALLOW_TAGS{$tagname}) {
	    $string .= "<$tagname>";
	}
	if ($tagname eq 'img') {
	    $string .= "[img]";
	}
    };

    my $_end_tag = sub {
	my ($tagname, $dtext) = @_;

	if (exists $ALLOW_TAGS{$tagname}) {
	    $string .= "</$tagname>";
	}
    };

    my $p = HTML::Parser->new(start_h => [$_start_tag, "tagname, dtext, attr"],
			      end_h => [$_end_tag, "tagname, dtext"],
			      text_h => [$_text, "dtext"],
			      );
    $p->parse($html);
    $p->eof;
    return $string;
}

1;
