package PureMessage::Observer;
use strict;
use File::Spec::Functions qw(catdir);


=head1 NAME

PureMessage::Observer - Allow modules to Observer Events 

=head1 SYNOPSIS

  use PureMessage::Observer;
  
  my $o = PureMessage::Observer->new('httpd');
  ..
  $o->notify('conf_changed',%changes);


=head1 DESCRIPTION

This package allows easy creation of observers. By placing a module
into the appropriate directory you can enable other Observer enabled 
code to notify you about "events". This is particularly handy when
you don't want to make a reverse dependance on a package.

To use place a package in the agreed upon directory under
C<PureMessage::Observer> (for example 'httpd') with the appropriate
event as a function. Now when the 'httpd' wants to notify all
interested parties of an 'event' it can just C<notify> your package
with the appropriate 'event'.

Note that there a few constraints on the observer packages you create:
they must have a valid perl package identifier for where they are
installed (e.g. C<PureMessage::Observer::httpd::watcher> if the
package is placed in the Observer/httdp/ directory). Also observer
types, packages names and file names are restricted to lowercase.

=item new($type)

Creates a new observer object for observers of C<$type>. This will
locate all observers of C<$type> by looking in all C<@inc> directories
for packages that have been installed under 
C<PureMessage::Observer::$type>.

=cut

sub new {
  my $class = shift;
  my $type = shift || '';
  my %observers = _get_observers($type);
  my $self = { observers => \%observers, type => $type,@_};
  return bless($self,$class);
}

sub _get_observers {
    my $type = shift || '';
    my %observers;
    for (@INC) {
        my $dir = catdir($_, split("::", __PACKAGE__),$type);
        next unless opendir(my $d, $dir);
        while (defined(my $f = readdir($d))) {
            next unless $f =~ /^([a-z][a-z_0-9]+)\.pm$/;
            $observers{$1} = 0;
        }
    }
    return %observers;
}

sub _type_method {
    my $self = shift;
    my $observer = shift;
    my $meth = shift;
    my @observer = $self->{'type'} ? ($self->{'type'}) : ();
    push(@observer,lc($observer));
    my $mod = join('::',__PACKAGE__,@observer);
    unless ($self->{observers}->{$observer}++) {
        eval "require $mod";
        die $@ if $@;
    }
    return $mod->$meth(@_);
}

sub _type_func {
    my $self = shift;
    my $observer = shift;
    my $func = shift;
    my @observer = $self->{'type'} ? ($self->{'type'}) : ();
    push(@observer,lc($observer));
    my $mod = join('::',__PACKAGE__,@observer);
    my %result = ( name => $observer, package => $mod );
    unless ($self->{observers}->{$observer}++) {
        eval "require $mod";
        if ($@) {
          $result{error} = $@;
          return \%result;
        }
    }
    my $fqf = $mod . '::' . $func; # what we are going to call
    
    # only notify if this module can accept it
    return unless $mod->can($func);
    
    my $value;
    eval {
        no strict "refs";
        $value = &$fqf(@_);
    };
    if ($@) {
      $result{error} = $@;
      return \%result;
    }
    $result{result} = $value;
    return \%result;
}

=item notify($event[,@args])

Notifies the observers that C<$event> happened. In actuality this
executes the function $event in each observer found, passing @args
to each. Observers are run in lexical order. The return of this method
is an array of hashes one each for each observer.  If there is an
error the C<error> key will be set, otherwise the C<result> of the
function call for that observer is set.

=cut

sub notify {
  my $self = shift;
  my $event = shift || 'notify';
  my @results;
  return unless $self->{observers} and keys %{$self->{observers}};
  push(@results,$self->_type_func($_,$event,@_)) foreach sort keys %{$self->{observers}};  
  return @results;
}


1;

