package PureMessage::Service;

use strict;
use File::Spec::Functions qw(catdir);

my %loaded;

for (@INC) {
    my $dir = catdir($_, split("::", __PACKAGE__));
    next unless opendir(my $d, $dir);
    while (defined(my $f = readdir($d))) {
	next unless $f =~ /^([a-z]\w+)\.pm$/;
	$loaded{$1} = 0;
    }
}

sub _type_method {
    my $type = shift;
    my $meth = shift;
    my $mod = __PACKAGE__ . "::" . lc($type);
    unless ($loaded{$type}++) {
	eval "require $mod";
	die $@ if $@;
    }
    return $mod->$meth(@_);
}

my %type_weight = ( milter => 0, smtp => 8, vfd => -10 );
my %name_weight = ( "milter.Policy" => 12, "milter.Main" => 10 );

sub _weight {
    my $s = shift;
    my $t = $s->type;
    my $w = $type_weight{$t} || 0;
    my $n = $s->name;
    if ($n && $name_weight{"$t.$n"}) {
	$w += $name_weight{"$t.$n"};
    }
    return $w;
}

sub _serv_cmp {
    _weight($b) <=> _weight($a);
}

sub Available {
    my $class = shift;
    return $class->new if $class ne __PACKAGE__;
    return sort _serv_cmp
           map _type_method($_, "Available"), keys %loaded;
}

sub new {
    my $class = shift;
    if ($class ne __PACKAGE__) {
	my %self = @_;
	return bless \%self, $class;
    }

    my $type = shift;
    return _type_method($type, "new", @_);
}

sub type {
    my $self = shift;
    (my $type = ref($self)) =~ s/.*:://;
    $type;
}

sub name {
    my $self = shift;
    my $old = $self->{name};
    $self->{name} = shift if @_;
    return $old;
}

sub status {
    my($self, %opts) = @_;
    return { running => 0 }
}
sub stale {
    my $self = shift;
    PureMessage::Service::Config::stale($self->type,$self->name);
}

sub ftime {
    my ($self,$file) = @_;
    my @res = stat($file);
    return unless @res;
    return $res[9];
}

sub running {
    my($self, %opts) = @_;
    return $self->status->{running};
}

1;


__END__

=head1 NAME

PureMessage::Service - Interface for discovery and control of daemons

=head1 SYNOPSIS

  use PureMessage::Service;

  @services = PureMessage::Service->Available;
  for my $s (@services) {
      next unless $s->type eq "milter";
      $s->stop(bg => 1) if $s->running;
  }

=head1 DESCRIPTION

The C<PureMessage::Service> module provide an interface for discovery
and control of service daemons.  The module provide a unified
interface for controlling all the different kinds of services provided
by the PureMessage system.

The following class/constructor methods are provided:

=over

=item @s = PureMessage::Service->Available

Returns a list of service objects, one for each enabled PureMessage
service on the system.

=item $s = PureMessage::Service->new( $type, %opts );

Constructs a new service object of the specified type.  Certain types
might have mandatory options.

=back

The following methods are available for a service object:

=over

=item $s->type

This returns the type of the service.  It is a short string like
"milter", "vfd" or "sendmail".

=item $s->name

This returns a name for the service.  For service types where there
can be multiple instances, each one will have a different name.  For
services where there can only be a single instance running, then this
attribute is C<undef>.

=item $s->start( %opts )

Will tell the service to start up.  The return value is true if the
service started up successfully.

If the C<bg> option is provided with a true value, then startup
happens in the background and the function will return the PID of the
background process.  Any output generated during startup is suppressed
under C<bg>.  The C<file> option can be provided to redirect all
output to the given file.

=item $s->stop( %opts )

Will tell the service to stop.  It supports the 'bg' option in a
similar way as start.

=item $s->restart( %opts )

Basically the same as $s->stop followed by $s->start.  It supports the
'bg' option in a similar way as start.

=item $s->stale()

Notifies the service that changes have been made to a config file that
it depends upon and it needs to be restarted.

=item $s->status( %opts )

Returns a hash reference that describes the status of the service.
Options can be provided to select more detailed information gathering.

Hash keys starting with lower case are reserved to have standard
meaning independently of the service type.  Standard status values
are:

=over

=item C<pid>

The main PID of the service.  If the service is implemented as
multiple cooperating processes, then this would be the root pid of the
process tree.

=item C<running>

A boolean that is TRUE if the service is currently up and running.
Undefined value if the status can't be identified.

=item C<ready>

A boolean that is TRUE iff the service is ready to service clients.

=item C<autostart>

A boolean that is TRUE iff the service will start up automatically on
system reboot.

=item C<config>

Value is one of C<sync>, C<stale> or C<gone>.

=item C<port>

If the service is listening on a port, then this list the contact
point.  The value is a milter style port spec, e.g.:

   inet:3366
   inet:3366@localhost
   unix:/tmp/.xxx.sock

=item C<start_time>

When was the service started up.  This is a time value (seconds since
epoch).  The service has then been running for C<time -
$status{start_time}>.

=item C<summary>

A short English text proving additional information or the current
state of the service.  Assume this text is presented together with the
type/name/running/pid/port fields, so there is no need to repeat that
information.

The summary text should normally be kept less than 50-60 chars long.

=back

=item $s->running

Returns true if the service is currently up and running.

=back
