package PureMessage::Template;

use strict;
use base 'Exporter';

our @EXPORT_OK = qw(expand_template extract_templates);

sub expand_template {
    my($text, %opts) = @_;
    $text =~ s/%%(\w+(?::\w+)*)%%/_expand_var($1, \%opts)/ge;
    return $text;
}

sub extract_templates {
    my ($text) = @_;
    my @vars = ();
    $text =~ s/%%(\w+)(?::\w+)*%%/push @vars, uc $1 unless grep $_ eq uc $1, @vars/ge;
    return @vars;
}

sub _expand_var {
    my $orig = shift;
    my($var, @mod) = split(/:/, $orig);
    my $lc_var = lc($var);
    my $opts = shift;

    my $value;

    if (defined(my $v = $opts->{vars}{$lc_var})) {
	if (ref($v) eq 'CODE') {
	    $value = &$v;
	}
	else {
	    $value = $v;
	}
    }
    else {
	no strict 'refs';
	my $var_func = "_var_" . lc($var);
	if (defined &$var_func) {
	    $value = &$var_func();
	}
    }

    unless (defined $value) {
	if (my $cb = $opts->{vars_callback}) {
	    $value = &$cb(uc $var);
	}
	else {
	    return "%%$orig%%";
	}
    }

    for my $mod (@mod) {
	no strict 'refs';
	my $mod_func = "_mod_" . lc($mod);
	if (defined &$mod_func) {
	    $value = &$mod_func($value);
	}
    }

    return $value;
}

my $hostname;

sub _var_hostname {
    return $hostname if defined $hostname;

    $hostname = `hostname`;
    chomp($hostname);
    $hostname;
}

sub _var_admin_addr {
    require PureMessage::Config;
    $PureMessage::Config::config{admin};
}

sub _mod_lower {
    lc(shift);
}

sub _mod_upper {
    uc(shift);
}

sub _mod_html {
    my $v = shift;
    $v =~ s/&/&amp;/g;
    $v =~ s/</&lt;/g;
    $v =~ s/>/&gt;/g;
    $v =~ s/\"/&quot;/g;
    $v;
}

sub _mod_uri {
    require URI::Escape;
    URI::Escape::uri_escape(shift);
}

sub _mod_ascii {
    my $v = shift;
    $v =~ s/[^\r\n\t\x20-\x7e]/?/g;
    $v;
}

sub _mod_base64 {
    require MIME::Base64;
    encode_base64(shift, "");
}

sub _mod_hex {
    unpack("H*", shift);
}

sub _mod_address {
    return _mod_mail_addr(@_);
}

sub _mod_mail_addr {
    require Mail::Address;
    local $Mail::Address::VERBOSE = 0; # suppress Mail::Address warnings
    my ($addr) = Mail::Address->parse(shift) or return '';
    $addr->address;
}

sub _mod_mail_name {
    require Mail::Address;
    local $Mail::Address::VERBOSE = 0; # suppress Mail::Address warnings
    my ($addr) = Mail::Address->parse(shift) or return '';
    $addr->name;
}

sub _mod_h_utf8 {
    my $h = shift;
    require PureMessage::UTF8::Config;
    require ActiveState::I18N::UTF8;
    eval {
        # Remove low-bit control characters
        $h =~ tr/\x00-\x1F//d;
        ActiveState::I18N::UTF8::utf8_header($h, \%PureMessage::UTF8::Config::utf8_config);
        # XXX - should we warn if we cannot convert?
    };
    return $h;
}

1;

=head1 NAME

PureMessage::Template - Expand template variables in text

=head1 SYNOPSIS

   use PureMessage::Template 'expand_template';

   # Hello, world!
   print expand_template(<<'TXT', vars => { greeting => 'Hello' });
   %%GREETING%%, world!
   TXT

=head1 DESCRIPTION

PureMessage::Template will be used to expand templates of the form C<%%FOO%%>
in various places for PureMessage 3.5. It optionally exports two functions:
expand_template(), which expands template variables in a chunk of text and
returns the modified text; and extract_templates(), which returns the name of
each template variable found in a chunk of text.

=head2 Extracting Template Variables

   @vars = extract_templates($template);

Parameters:

=over 4

=item $template

The template text. All strings matching the regular expression
C</\%\%\w+(?::\w+)*\%\%/> are returned, minus the surrounding C<%%>. They are
always returned in UPPERCASE, regardless of how they appeared in the template.

=back

=head2 Expanding Templates

   $text = expand_template($template, %opts);

Parameters:

=over 4

=item $template

The template text. All strings matching the regular expression
C</\%\%\w+(?::\w+)*\%\%/> are substituted. The name of the template itself is
the initial C<\w+>; the name is followed by optional modifiers.

For example, the following prints the administrator's address escaped in HTML:

   print expand_template("%%ADMIN_ADDR:html%%\n");

=item %opts

The following options are supported:

=over 4

=item vars

A hash reference containing key/value pairs. Each key is the lowercase name of
a template variable, and each value is the text to be substituted. If the
value is a CODE reference, it will be evaluated, and its return value will be
used as the value.

=item vars_callback

If an unknown template is seen, and I<vars_callback> is defined, it will be
called with the name of the template in UPPERCASE. Its return value will be
used as the substitute for the template. If the callback is not provided, the
original string will be left alone.

=item message - I<NOT IMPLEMENTED>

A PureMessage::Message object. If it is defined, it provides the following
template variables:

=over 4

=item HEADER_x

A header of the original email. For example, C<%%HEADER_FROM%%> returns the
"From" header.

=item SUBJECT_UTF8

Similar to C<%%HEADER_SUBJECT%%>, but text is unwrapped and RFC 2047
encoding undone as UTF-8.

=item SUBJECT_ASCII

Similar to SUBJECT_UTF8, but chars not within 7-bit US-ASCII range are
replaced by a "?".

=item ENVELOPE_x

The envelope-from or envelope-to of the original email. For example,
C<%%ENVELOPE_TO%%> returns the envelope-to address.

=back

=back

=back

=head2 Default Templates

By default, expand_template() always knows about the following templates:

=over 4

=item ADMIN_ADDR

The name and email address of the PureMessage administrator (from pmx.conf).

=item HOSTNAME

The current hostname.

=back

=head2 Format Modifiers

Format modifiers change the way the text is transformed before being inserted
into the original template. Formatters can be chained to together, in which
case they apply from left to right. Each formatter sees the output of the
previous, with the first seeing the raw text.

The following format modifiers are supported:

=over 4

=item html

The output is escaped in HTML format. For example:

   expand_template("%%ADMIN_ADDR:html%%");

returns the text

   PureMessage Administrator &lt;pmxadmin@example.com&gt;

=item uri

The output is escaped in URI format. All non-word characters are escaped. For
example:

   expand_template("%%ADMIN_ADDR:uri%%");

returns the text

   PureMessage%20Administrator%20%3Cpmxadmin%40example.com%3E

=item upper

The output is transformed to UPPERCASE. For example:

   expand_template("%%ADMIN_ADDR:upper%%");

returns the text

   PUREMESSAGE ADMINISTRATOR <PMXADMIN@EXAMPLE.COM>

=item lower

The output is transformed to lowercase. For example:

   expand_template("%%FOO:lower%%", vars => { foo => "BAR" });

returns the text

   bar

=item ascii

The output is transformed to ASCII. Any non-ASCII characters are replaced with
a C<?> character.

=item base64

The output is encoded in Base-64 format. For example:

   expand_template("%%ADMIN_ADDR:base64%%");

returns the text

   UHVyZU1lc3NhZ2UgQWRtaW4gPHBteGFkbWluLmNvbT4=

=item hex

The output is encoded in Hex format. For example:

   expand_template("%%ADMIN_ADDR:hex%%");

returns the text

   507572654d6573736167652041646d696e203c706d7861646d696e2e636f6d3e

=item address

=item mail_addr

The value is parsed with Mail::Address, and only the 'address' part is
returned. For example:

   expand_template("%%ADMIN_ADDR:mail_address%%");

returns the text

   pmxadmin@example.com

=item mail_name

The value is parsed with Mail::Address, and only the 'name' part is
returned. For example:

   expand_template("%%ADMIN_ADDR:mail_name%%");

returns the text

   PureMessage Administrator

=item h_utf8

The value is treated as an email header and decoded into UTF-8. Specifically,
RFC 2047 entities are decoded.

=back

=head1 EXAMPLES

Here are some examples.

=head2 Undefined Template

This code snippet:

   $txt = expand_template(<<'TMPL');
   This is undefined: %%UNDEF%%
   TMPL

produces this text: C<This is undefined: %%UNDEF%%\n>.

=head2 Using a Message - I<NOT IMPLEMENTED>

This code snippet:

   $txt = expand_template(<<'TMPL', message => $message);
   This is the sender: %%ENVELOPE_FROM%%
   TMPL

produces this text: C<This is the sender: joe@example.com\n>, assuming
that the $message came from joe@example.com.

=head2 Custom Template Variables

This code snippet:

   $txt = expand_template(<<'TMPL', vars => { fiddle => 'sticks' });
   %%FIDDLE%% and stones...
   TMPL

produces this text: C<stick and stones...\n>.

=head2 Handling Errors Yourself

This code snippet:

   sub foo { "snippet" }
   $txt = expand_template(<<'TMPL', vars_callback => \&foo);
   This is a %%SNIPPET%%
   TMPL

produces this text: C<This is a snippet\n>.

=head2 Put it all Together

This code snippet:

   sub errmsg {
       my $var = shift;
       "Undefined template variable %%$var%%";
   }
   %vars = (
       hostname => sub { gethostname() },
       time     => sub { scalar localtime },
       company  => 'Wonderland, Inc.',
       challenge => 'What is the capital of Spain?',
   );
   $template = <<'TMPL';
   Hi %%HEADER_FROM%%,

   It looks like you tried to send an email to us:

      To: %%HEADER_TO%%
      Subject: %%HEADER_SUBJECT%%
      Date: %%HEADER_DATE%%

   Unfortunately, you're not on our approved sender list, so you need to
   answer the following question:

      %%CHALLENGE%%

   If you're %%SURE%% you know the answer, email us back immediately!

   Later,
   %%COMPANY%%
   TMPL
   $txt = expand_template($template,
                          message => $message, 
                          vars => \%vars,
                          vars_callback => \&errmsg,
                         );

Produces this output (just believe be about the $message values):

   Hi Bob <bob@marley.com>,

   It looks like you tried to send an email to us:

      To: Alice <alice@wonderland.com>
      Subject: Check out my wristwatch!!!
      Date: Mon, 17 Feb 2003 15:48:31 -0800

   Unfortunately, you're not on our approved sender list, so you need to
   answer the following question:

      What is the capital of Spain?

   If you're Undefined template variable %%SURE%% you know the answer, email us back immediately!!!

   Later,
   Wonderland, Inc.

=head1 COPYRIGHT

Copyright (C) 2003, ActiveState Corporation. All Rights Reserved.

=cut
